/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2018-2021 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Application
    cipsaMultiphaseEulerFoam

Description
    Solver for a system of any number of incompressible fluid phases with a
    common pressure, but otherwise separate properties. The solver uses the
    consistent momentum interpolation method of Cubero et al. (2014) and a
    partial-elimination algorithm.

    Reference:
    \verbatim
        Cubero, A., Sánchez-Insa, A., & Fueyo, N. (2014).
        A consistent momentum interpolation method for steady and unsteady
        multiphase flows.
        Computers & Chemical Engineering, 62, 96-107.
    \endverbatim

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "phaseSystem.H"
#include "phaseCompressibleMomentumTransportModel.H"
#include "pimpleControl.H"
#include "initZeroPtrList.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{
    #include "postProcess.H"

    #include "setRootCaseLists.H"
    #include "createTime.H"
    #include "createMesh.H"
    #include "createControl.H"
    #include "createPIDTimeControls.H"
    #include "createFields.H"
    #include "createFieldRefs.H"
    #include "CourantNo.H"
    #include "setInitialPIDDeltaT.H"

    Switch faceMomentum
    (
        pimple.dict().lookupOrDefault<Switch>("faceMomentum", false)
    );

    Switch partialElimination
    (
        pimple.dict().lookupOrDefault<Switch>("partialElimination", true)
    );

    Switch energyPartialElimination
    (
        pimple.dict().lookupOrDefault<Switch>("energyPartialElimination", false)
    );

    Switch continuousBuoyancy
    (
        pimple.dict().lookupOrDefault<Switch>("continuousBuoyancy", false)
    );

    // * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    Info<< "\nStarting time loop\n" << endl;

    while (pimple.run(runTime))
    {
        #include "readPIDTimeControls.H"

        int nEnergyCorrectors
        (
            pimple.dict().lookupOrDefault<int>("nEnergyCorrectors", 1)
        );

        #include "CourantNo.H"
        #include "setPIDDeltaT.H"

        runTime++;

        Info<< "Time = " << runTime.timeName() << nl << endl;

        // --- Pressure-velocity corrector loop (PIMPLE)
        while (pimple.loop())
        {
            if (pimple.frozenFlow())
            {
                fvModels.correct();

                fluid.solve(rAUs, rHAUfs);
                fluid.correct();
                fluid.correctContinuityError();

                #include "EEqns.H"
                #include "pEqnComps.H"

                forAll(phases, phasei)
                {
                    phases[phasei].divU(-pEqnComps[phasei] & p_rgh);
                }
            }
            else
            {
                fvModels.correct();

                fluid.solve(rAUs, rHAUfs);
                fluid.correct();
                // Update continuityError in MovingPhaseModel for heEqn in
                // AnisothermalPhaseModel
                fluid.correctContinuityError();

                #include "pU/UEqns.H"
                #include "EEqns.H"

                // --- Pressure corrector loop (PISO)
                while (pimple.correct())
                {
                    #include "pU/pEqn.H"
                }

                // Update material derivatives
                fluid.correctKinematics();

                if (pimple.turbCorr())
                {
                    fluid.correctTurbulence();
                }
            }
        }

        runTime.write();

        Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
            << "  ClockTime = " << runTime.elapsedClockTime() << " s"
            << nl << endl;
    }

    Info<< "End\n" << endl;

    return 0;
}


// ************************************************************************* //
