/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2020-2021 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "degassingPhases.H"
#include "fvMatrix.H"
#include "fvmSup.H"
#include "surfaceInterpolate.H"
#include "addToRunTimeSelectionTable.H"
#include "cyclicAMIFvPatch.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace fv
{
    defineTypeNameAndDebug(degassingPhases, 0);
    addToRunTimeSelectionTable
    (
        fvModel,
        degassingPhases,
        dictionary
    );
}
}


// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //

bool Foam::fv::degassingPhases::isDegassingPhase(const word& fieldName) const
{
    return IOobject::group(fieldName) != continuousPhase_;
}


template<class Type>
void Foam::fv::degassingPhases::addSupType
(
    const volScalarField& alpha,
    fvMatrix<Type>& eqn,
    const word& fieldName
) const
{
    const GeometricField<Type, fvPatchField, volMesh>& psi = eqn.psi();
    const scalar deltaT = mesh().time().deltaT().value();
    const word alphaName = IOobject::member(alpha.name());

    volScalarField::Internal Sp
    (
        IOobject
        (
            name() + ":" + fieldName + ":Sp",
            mesh().time().timeName(),
            mesh()
        ),
        mesh(),
        dimensionedScalar(dimless/dimTime, Zero),
        false
    );

    if (isDegassingPhase(fieldName))
    {
        forAll(cells_, i)
        {
            Sp[cells_[i]] -= alpha[cells_[i]]/deltaT;
        }
    }
    else
    {
        dimensionedScalar volumeRate(dimensionSet(0, 3, -1, 0, 0), 0);
        dimensionedScalar boundaryRate(dimensionSet(0, 3, -1, 0, 0), 0);

        forAll(degassingPhases_, namei)
        {
            const surfaceScalarField& alphaRhoPhi =
                mesh().lookupObject<surfaceScalarField>
                (
                    IOobject::groupName
                    (
                        alphaName + "RhoPhi",
                        degassingPhases_[namei]
                    )
                );

            const volScalarField rhoAlpha
            (
                mesh().lookupObject<volScalarField>
                (
                    IOobject::groupName
                    (
                        "thermo:rho", degassingPhases_[namei]
                    )
                )
            );

            surfaceScalarField rhoAlphaf(fvc::interpolate(rhoAlpha));

            forAll(mesh().boundary(), patchi)
            {
                if
                (
                    mesh().boundary()[patchi].coupled()
                 || isA<cyclicAMIFvPatch>(mesh().boundary()[patchi])
                )
                {
                    continue;
                }

                boundaryRate +=
                    dimensionedScalar
                    (
                        dimensionSet(0, 3, -1, 0, 0),
                        sum
                        (
                            alphaRhoPhi.boundaryField()[patchi]
                           /rhoAlphaf.boundaryField()[patchi]
                        )
                    );
            }

            const fvModels& fvModels(fvModels::New(mesh()));
            const PtrListDictionary<fvModel>& modelList = (fvModels);

            forAll(modelList, i)
            {
                const fvModel& model = modelList[i];

                if
                (
                    model.name() != this->name()
                 && model.addsSupToField(rhoAlpha.name())
                )
                {
                    volScalarField source
                    (
                        (model.source(alpha, rhoAlpha) & rhoAlpha).ref()
                    );

                    volumeRate -= sum(source.ref()*mesh().V()/rhoAlpha);
                }
            }
        }

        reduce(boundaryRate, sumOp<dimensionedScalar>());

        scalar rate = (volumeRate.value() + boundaryRate.value())/V_;

        forAll(cells_, i)
        {
            Sp[cells_[i]] = rate;

            forAll(degassingPhases_, namei)
            {
                const volScalarField& degassingAlpha =
                    mesh().lookupObject<volScalarField>
                    (
                        IOobject::groupName(alphaName, degassingPhases_[namei])
                    );

                Sp[cells_[i]] +=
                    degassingAlpha[cells_[i]]/deltaT;
            }
        }
    }

    if (debug)
    {
        Info<< name() <<": " << fieldName << ": Sp min, max: "
            << min(Sp).value() << ", " << max(Sp).value() << endl;

        if (mesh().time().writeTime()) Sp.write();
    }

    eqn += fvm::SuSp(Sp, psi);
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::fv::degassingPhases::degassingPhases
(
    const word& name,
    const word& modelType,
    const dictionary& dict,
    const fvMesh& mesh
)
:
    fvModel(name, modelType, dict, mesh),
    excludePhases_
    (
        dict.lookupOrDefault<wordList>
        (
            "excludePhases",
            wordList(0)
        )
    ),
    fluid_(mesh.lookupObject<phaseSystem>("phaseProperties")),
    patchNames_(dict.lookup("patchNames")),
    V_(Zero)
{
    read(dict);

    if (fluid_.continuousPhases().size() != 1)
    {
        FatalErrorInFunction
            << "Exactly one continuous phase is required, but there are "
            << fluid_.continuousPhases().size() << " specified."
            << endl << exit(FatalError);
    }

    forAll(fluid_.movingPhases(), phasei)
    {
        const phaseModel& phase = fluid_.movingPhases()[phasei];
        word rhoName = IOobject::groupName("thermo:rho", phase.name());

        if (pos0(findIndex(excludePhases_, phase.name())))
        {
            continue;
        }
        else
        {
            fieldNames_.append(rhoName);
        }

        if (phase.continuous())
        {
            continuousPhase_ = phase.name();
        }
        else
        {
            degassingPhases_.append(phase.name());
        }
    }

    forAll(patchNames_, patchi)
    {
        const fvPatch& patchField =
            mesh.boundary()[patchNames_[patchi]];

        forAll(patchField.faceCells(), i)
        {
            cells_.append(patchField.faceCells()[i]);
            V_ += mesh.V()[patchField.faceCells()[i]];
        }
    }
    reduce(V_, sumOp<scalar>());
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::fv::degassingPhases::~degassingPhases()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

bool Foam::fv::degassingPhases::addsSupToField(const word& fieldName) const
{
    return pos0(findIndex(fieldNames_, fieldName));
}


Foam::wordList Foam::fv::degassingPhases::addSupFields() const
{
    return fieldNames_;
}


FOR_ALL_FIELD_TYPES
(
    IMPLEMENT_FV_MODEL_ADD_RHO_SUP,
    fv::degassingPhases
);


// ************************************************************************* //
