/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2019-2021 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "ReynoldsDependentWeberNumber.H"
#include "phasePair.H"
#include "addToRunTimeSelectionTable.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace subGridSurfaceTensionModels
{
    defineTypeNameAndDebug(ReynoldsDependentWeberNumber, 0);
    addToRunTimeSelectionTable
    (
        subGridSurfaceTensionModel,
        ReynoldsDependentWeberNumber,
        dictionary
    );
}
}


// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::subGridSurfaceTensionModels::ReynoldsDependentWeberNumber::nu() const
{
    const phaseModel& phase1 = pair().phase1();
    const phaseModel& phase2 = pair().phase2();

    return
        phase1*phase1.thermo().nu() + phase2*phase2.thermo().nu()
       /max
        (
            phase1 + phase2,
            phase1.residualAlpha() + phase2.residualAlpha()
        );
}


Foam::tmp<Foam::volScalarField>
Foam::subGridSurfaceTensionModels::ReynoldsDependentWeberNumber::ReLocal
() const
{
    return max(mag(U())*delta()/nu(), residualRe_);
}


Foam::tmp<Foam::volScalarField>
Foam::subGridSurfaceTensionModels::ReynoldsDependentWeberNumber::
WeCrit() const
{
    tmp<volScalarField> WeCrit
    (
        volScalarField::New
        (
            IOobject::groupName("WeCrit", pair().name()),
            pow
            (
                0.32366893582752687
              + 4.729129024951329*pow(ReLocal(), -5.0/6.0),
               -0.9
            )
        )
    );

    if (debug && mesh().time().outputTime())
    {
        WeCrit.ref().write();
    }

    return WeCrit;
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::subGridSurfaceTensionModels::ReynoldsDependentWeberNumber::
ReynoldsDependentWeberNumber
(
    const dictionary& dict,
    const phasePair& pair
)
:
    WeberNumberModel(dict, pair),

    residualRe_
    (
        dimensioned<scalar>::lookupOrAddToDict
        (
            "residualRe",
            this->coeffDict_,
            1e-3
        )
    )
{
    if (type() == typeName)
    {
        this->printCoeffs(type());
    }
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::subGridSurfaceTensionModels::ReynoldsDependentWeberNumber::
~ReynoldsDependentWeberNumber()
{}


// ************************************************************************* //
