/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2019-2021 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "WeberNumberModel.H"
#include "phasePair.H"

// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::subGridSurfaceTensionModels::WeberNumberModel::rho() const
{
    return pair().rho();
}


Foam::tmp<Foam::volScalarField>
Foam::subGridSurfaceTensionModels::WeberNumberModel::sigmaEff
(
    const volScalarField& K
) const
{
    return this->sigma() + this->sigmat(K);
}


Foam::tmp<Foam::volVectorField>
Foam::subGridSurfaceTensionModels::WeberNumberModel::subGridSurfaceTensionForce
(
    const volScalarField& K
) const
{
    return this->sigmat(K)*K*gradAlpha();
}


// * * * * * * * * * * * * Protected Member Functions  * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::subGridSurfaceTensionModels::WeberNumberModel::sigmat
(
    const volScalarField& K
) const
{
    tmp<volScalarField> sigmaCrit
    (
        rho()*magSqr(U())*sqr(delta())*mag(K)/this->WeCrit()
    );

    return
        max
        (
            sigmaCrit - this->sigma(),
            dimensionedScalar(this->sigma()().dimensions(), Zero)
        );
}


Foam::tmp<Foam::surfaceScalarField>
Foam::subGridSurfaceTensionModels::WeberNumberModel::sigmatf
(
    const volScalarField& K
) const
{
    tmp<surfaceScalarField> sigmaCritf
    (
        fvc::interpolate(rho())*fvc::interpolate(magSqr(U()))
       *magSqr(mesh().delta())*fvc::interpolate(mag(K))
       /fvc::interpolate(this->WeCrit())
    );

    return
        max
        (
            sigmaCritf - fvc::interpolate(this->sigma()),
            dimensionedScalar(this->sigma()().dimensions(), Zero)
        );
}


Foam::tmp<Foam::volVectorField>
Foam::subGridSurfaceTensionModels::WeberNumberModel::U() const
{
    const phaseModel& phase1 = pair().phase1();
    const phaseModel& phase2 = pair().phase2();

    return
        phase1*phase1.U() + phase2*phase2.U()
       /max
        (
            phase1 + phase2,
            phase1.residualAlpha() + phase2.residualAlpha()
        );
}


bool Foam::subGridSurfaceTensionModels::WeberNumberModel::write
(
    const volScalarField& K
) const
{
    if (subGridSurfaceTensionModel::write(K))
    {
        tmp<volScalarField> sigmaEffective
        (
            volScalarField::New
            (
                IOobject::groupName("sigmaEff", pair().name()),
                sigmaEff(K)
            )
        );
        sigmaEffective.ref().write();

        return true;
    }
    else
    {
        return false;
    }
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::subGridSurfaceTensionModels::WeberNumberModel::WeberNumberModel
(
    const dictionary& dict,
    const phasePair& pair
)
:
    subGridSurfaceTensionModel(dict, pair),

    delta_
    (
        SGSdelta::New
        (
            IOobject::groupName("delta", pair.name()),
            mesh(),
            dict
        )
    )
{}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::subGridSurfaceTensionModels::WeberNumberModel::~WeberNumberModel()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

Foam::tmp<Foam::surfaceScalarField>
Foam::subGridSurfaceTensionModels::WeberNumberModel::subGridSurfaceTensionForcef
(
    const volScalarField& K
) const
{
    if (debug && mesh().time().outputTime())
    {
        this->write(K);
    }

    return this->sigmatf(K)*fvc::interpolate(K)*gradAlphaf();
}


// ************************************************************************* //
