/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2019-2020 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "subGridSurfaceTensionModel.H"
#include "phasePair.H"
#include "phaseSystem.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
    defineTypeNameAndDebug(subGridSurfaceTensionModel, 0);
    defineRunTimeSelectionTable(subGridSurfaceTensionModel, dictionary);
}


// * * * * * * * * * * * * Protected Member Functions  * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::subGridSurfaceTensionModel::sigma() const
{
    return pair().phase1().fluid().sigma(pair());
}


Foam::tmp<Foam::volVectorField>
Foam::subGridSurfaceTensionModel::gradAlpha() const
{
    const volScalarField& phase1 = pair().phase1();
    const volScalarField& phase2 = pair().phase2();

    return phase2*fvc::grad(phase1) - phase1*fvc::grad(phase2);
}


Foam::tmp<Foam::surfaceScalarField>
Foam::subGridSurfaceTensionModel::gradAlphaf() const
{
    const volScalarField& phase1 = pair().phase1();
    const volScalarField& phase2 = pair().phase2();

    return
        fvc::interpolate(phase2)*fvc::snGrad(phase1)
      - fvc::interpolate(phase1)*fvc::snGrad(phase2);
}


void Foam::subGridSurfaceTensionModel::printCoeffs(const word& type)
{
    if (printCoeffs_)
    {
        Info<< coeffDict_.dictName() << coeffDict_ << endl;
    }
}


bool Foam::subGridSurfaceTensionModel::write
(
    const volScalarField& K
) const
{
    tmp<volVectorField> sgsSt
    (
        volVectorField::New
        (
            IOobject::groupName
            (
                "subGridSurfaceTensionForce",
                pair().name()
            ),
            this->subGridSurfaceTensionForce(K)
        )
    );
    sgsSt.ref().write();

    return true;
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::subGridSurfaceTensionModel::subGridSurfaceTensionModel
(
    const dictionary& dict,
    const phasePair& pair
)
:
    mesh_(pair.phase1().mesh()),
    pair_(pair),
    printCoeffs_(dict.lookupOrDefault<Switch>("printCoeffs", false)),
    coeffDict_(dict.optionalSubDict(type() + "Coeffs"))
{}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::subGridSurfaceTensionModel::~subGridSurfaceTensionModel()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

Foam::tmp<Foam::surfaceScalarField>
Foam::subGridSurfaceTensionModel::subGridSurfaceTensionForcef
(
    const volScalarField& K
) const
{
    if (debug && mesh().time().outputTime())
    {
        this->write(K);
    }

    return fvc::flux(this->subGridSurfaceTensionForce(K))/mesh().magSf();
}


// ************************************************************************* //
