/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2019-2020 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::subGridSurfaceTensionModel

Description
    Base class for modeling the sub-grid surface tension flux.

SourceFiles
    subGridSurfaceTensionModel.C
    subGridSurfaceTensionModelNew.C

\*---------------------------------------------------------------------------*/

#ifndef subGridSurfaceTensionModel_H
#define subGridSurfaceTensionModel_H

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#include "volFields.H"
#include "dictionary.H"
#include "runTimeSelectionTables.H"
#include "fvc.H"

namespace Foam
{

class phasePair;
class phaseModel;

/*---------------------------------------------------------------------------*\
                 Class subGridSurfaceTensionModel Declaration
\*---------------------------------------------------------------------------*/

class subGridSurfaceTensionModel
{
    // Private data

        //- Reference to the mesh
        const fvMesh& mesh_;

        //- Phase pair
        const phasePair& pair_;

        //- Flag to print the model coeffs at run-time
        Switch printCoeffs_;


    // Private member functions

        //- Sub grid surface tension force
        virtual tmp<volVectorField> subGridSurfaceTensionForce
        (
            const volScalarField& K
        ) const = 0;


protected:
    // Protected data

        //- Model coefficients dictionary
        dictionary coeffDict_;

    // Protected member functions

        // Access

            //- Const access to mesh
            const fvMesh& mesh() const
            {
                return mesh_;
            }


        //- Return the surface tension of this pair
        tmp<volScalarField> sigma() const;

        //- Interface gradient of phase pair
        tmp<volVectorField> gradAlpha() const;

        //- Face based interface gradient of phase pair
        tmp<surfaceScalarField> gradAlphaf() const;

        //- Print model coefficients
        void printCoeffs(const word& type);

        //- Write out effective sub-grid surface tension force
        virtual bool write
        (
            const volScalarField& K
        ) const;


public:

    //- Runtime type information
    TypeName("subGridSurfaceTensionModel");


    // Declare runtime construction

        declareRunTimeSelectionTable
        (
            autoPtr,
            subGridSurfaceTensionModel,
            dictionary,
            (
                const dictionary& dict,
                const phasePair& pair
            ),
            (dict, pair)
        );


    // Constructors

        //- Construct from a dictionary and a phase pair
        subGridSurfaceTensionModel
        (
            const dictionary& dict,
            const phasePair& pair
        );


    //- Destructor
    virtual ~subGridSurfaceTensionModel();


    // Selectors

        static autoPtr<subGridSurfaceTensionModel> New
        (
            const dictionary& dict,
            const phasePair& pair
        );


    // Member Functions

        // Access

            //- Const access to phase pair
            const phasePair& pair() const
            {
                return pair_;
            }


        //- Sub grid surface tension flux
        virtual tmp<surfaceScalarField> subGridSurfaceTensionForcef
        (
            const volScalarField& K
        ) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
