/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2018-2020 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::aspectRatioModels::ZiegenheinLucas

Description
    Aspect ratio model of Ziegenhein and Lucas which uses the functional form of
    Wellek et al. but with a different coefficient A and exponent B.

    \f[
        d_{\perp} = d_b  \sqrt[3]{1 + A \mathrm{Eo}^{B}}
    \f]

    where

    \vartable
        d_{\perp}   | Bubble diameter perpendicular to main motion
        d_b         | Bulk bubble diameter
        \mathrm{Eo} | Eötvös number
    \endvartable

    Both values depend on the degree of contimination and the involved
    substances and are therefore user-modifiable. The default values set here
    are not explicitly stated by Ziegenhein et. al, but can be extracted from
    the experimental data.

    References:
    \verbatim
        Ziegenhein, T., & Lucas, D. (2017).
        Observations on bubble shapes in bubble columns under different flow
        conditions.
        Experimental Thermal and Fluid Science, 85, 248-256.
    \endverbatim

    \verbatim
        Wellek, R. M., Agrawal, A. K., & Skelland, A. H. P. (1966).
        Shape of liquid drops moving in liquid media.
        AIChE Journal, 12(5), 854-862.
    \endverbatim

Usage
    \table
        Property     | Description             | Required    | Default value
        A            | Coefficient A           | no          | 0.65
        B            | Coefficient B           | no          | 0.35
    \endtable

SourceFiles
    ZiegenheinLucas.C

\*---------------------------------------------------------------------------*/

#ifndef ZiegenheinLucas_H
#define ZiegenheinLucas_H

#include "aspectRatioModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace aspectRatioModels
{

/*---------------------------------------------------------------------------*\
                       Class ZiegenheinLucas Declaration
\*---------------------------------------------------------------------------*/

class ZiegenheinLucas
:
    public aspectRatioModel
{
    // Private Data

        //- Optional coefficient A, defaults to 0.65
        scalar A_;

        //- Optional exponent B, defaults to 0.35
        scalar B_;


public:

    //- Runtime type information
    TypeName("ZiegenheinLucas");


    // Constructors

        //- Construct from a dictionary and an ordered phase pair
        ZiegenheinLucas
        (
            const dictionary& dict,
            const phasePair& pair
        );


    //- Destructor
    virtual ~ZiegenheinLucas();


    // Member Functions

        //- Aspect ratio
        virtual tmp<volScalarField> E() const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace aspectRatioModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
