/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2019-2020 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "detrainment.H"
#include "phasePair.H"
#include "phaseSystem.H"
#include "addToRunTimeSelectionTable.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace phaseTransferModels
{
    defineTypeNameAndDebug(detrainment, 0);
    addToRunTimeSelectionTable(phaseTransferModel, detrainment, dictionary);
}
}


// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::phaseTransferModels::detrainment::rate(
    const phaseModel& pairDisperse,
    const phaseModel& pairContinuous
) const
{
    tmp<volScalarField> tRate
    (
        volScalarField::New
        (
            IOobject::groupName("dmdtf."+typeName, pair_.name()),
            pairContinuous.mesh(),
            dimensionedScalar(dimDensity/dimTime, Zero)
        )
    );

    tRate.ref() =
        pairDisperse.rho()/(nDeltaT_*pairDisperse.fluid().time().deltaT());

    return tRate;
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::phaseTransferModels::detrainment::detrainment
(
    const dictionary& dict,
    const phasePair& pair
)
:
    phaseTransferModel(dict, pair),
    pair_(pair),
    continuousPhases_(pair_.phase1().fluid().continuousPhases()),
    nDeltaT_(dict.lookupOrDefault<scalar>("nDeltaT", 100)),
    pairContinuousRatioThreshold_
    (
        dimensionedScalar::lookupOrDefault
        (
            "pairContinuousRatioThreshold",
            dict,
            dimless,
            0.9
        )
    )
{
    if ((!pair_.phase1().continuous()) && (!pair_.phase2().continuous()))
    {
        FatalErrorInFunction
            << "A continuous phase is expected in the pair "
            << pair_.name()
            << " for detrainment modeling"
            << exit(FatalError);
    }
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::phaseTransferModels::detrainment::~detrainment()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

bool Foam::phaseTransferModels::detrainment::mixture() const
{
    return true;
}


Foam::tmp<Foam::volScalarField>
Foam::phaseTransferModels::detrainment::dmdtf() const
{
    tmp<volScalarField> tDmdtf
    (
        volScalarField::New
        (
            IOobject::groupName("dmdtf."+typeName, pair_.name()),
            pair_.phase1().mesh(),
            dimensionedScalar(dimDensity/dimTime, Zero)
        )
    );

    const phaseModel* pairDispersePtr = nullptr;
    const phaseModel* pairContinuousPtr = nullptr;

    if (pair_.phase1().continuous())
    {
        pairDispersePtr = &pair_.phase2();
        pairContinuousPtr = &pair_.phase1();
    }
    else
    {
        pairDispersePtr = &pair_.phase1();
        pairContinuousPtr = &pair_.phase2();
    }

    const phaseModel& pairDisperse = *pairDispersePtr;
    const phaseModel& pairContinuous = *pairContinuousPtr;
    const scalar sign = pair_.phase1().continuous() ? 1 : -1;

    tDmdtf.ref() = sign*rate(pairDisperse, pairContinuous);

    tDmdtf.ref() *=
        pos0
        (
            pairContinuous
           /max(pairDisperse.fluid().sumAlphaContinuous(), small)
          - pairContinuousRatioThreshold_
        )
       *pos(pairDisperse - pairDisperse.residualAlpha());

    if (debug && pair_.phase1().time().outputTime())
    {
        tDmdtf.ref().write();
    }

    return tDmdtf;
}


// ************************************************************************* //
