/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2019-2020 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::phaseTransferModels::detrainment

Description
    Phase transfer model representing change from a dispersed phase to a
    continuous phase with identical physical quantities.

    The keyword 'pairContinuousRatioThreshold' is the minimum value of the
    ratio of the continuous phase volume fraction of the specified phasePair
    to the sum of all continuous phase volume fractions. It defines where
    detrainment is active; i.e. detrainment is active in cells with:

    \f[
        \frac{\alpha_{pairCont}}
             {\sum_{i \in cont} \alpha_{i}}
        >
        pairContinuousRatioThreshold
    \f]
    The value will limit the detrainment to take place only at on side
    of an interface (ratio > 0.X).

Usage
    \table
        Property    | Description                          | Required | Default
        nDeltaT     | Number of time steps for complete transfer   | no   | 100
        pairContinuousRatioThreshold | Minimum continuous fraction | no   | 0.9
    \endtable

SourceFiles
    detrainment.C

\*---------------------------------------------------------------------------*/

#ifndef detrainment_H
#define detrainment_H

#include "phaseTransferModel.H"
#include "phaseModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

class phasePair;

namespace phaseTransferModels
{

/*---------------------------------------------------------------------------*\
                         Class detrainment Declaration
\*---------------------------------------------------------------------------*/

class detrainment
:
    public phaseTransferModel
{
private:

    // Private typedef

        typedef UPtrList<phaseModel> phaseModelPartialList;


    // Private Data

        //- Phase pair
        const phasePair& pair_;

        //- Name list of the continuous phases
        const phaseModelPartialList continuousPhases_;

        //- Number of time steps to completely transfer volume fraction
        const scalar nDeltaT_;

        //- Threshold for volume fraction of pairContinuous phase
        const dimensionedScalar pairContinuousRatioThreshold_;


    // Private member functions

        //- Return the detrainment rate
        Foam::tmp<volScalarField> rate(
            const phaseModel& pairDisperse,
            const phaseModel& pairContinuous
        ) const;


public:

    //- Runtime type information
    TypeName("detrainment");


    // Constructors

        //- Construct from components
        detrainment
        (
            const dictionary& dict,
            const phasePair& pair
        );


    //- Destructor
    virtual ~detrainment();


    // Member Functions

        //- Whether or not there is a mass transfer rate for the mixture
        virtual bool mixture() const;

        //- The mass transfer rate for the mixture
        virtual tmp<volScalarField> dmdtf() const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace phaseTransferModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
