/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2020 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::RASModels::omegaEgorovRASModel

Description
    Interfacial turbulence damping model for standard high Reynolds-number
    k-omega turbulence model for incompressible and compressible flows according
    to Egorov (2004) for stratified flows. The formulation of
    Frederix et al. (2018) is used, considering the damping length scale
    \f$\delta = \Delta n^2/6B\f$.

    References:
    \verbatim
        Egorov, Y., Boucker, M., Martin, A., Pigny, S., Scheuerer, M.,
        & Willemsen, S. (2004).
        Validation of CFD codes with PTS-relevant test cases.
        5th Euratom Framework Programme ECORA project, 91-116.

        Frederix, E. M. A., Mathur, A., Dovizio, D., Geurts, B. J.,
        & Komen, E. M. J. (2018).
        Reynolds-averaged modeling of turbulence damping near a large-scale
        interface in two-phase flow. Nucl. Eng. Des., 333, 122–130.
    \endverbatim

    Optional turbulence damping parameter
        \table
            Name  | Description           | Default
            delta | Damping scale     [m] | 1e-4
        \endtable

Usage
    Turbulence damping requires specification of interface indicator
    via the interfaceIndicatorMethod entry in constant/phaseProperties.

    Example with gradAlpha detection method:
    \verbatim
    interfaceIndicatorMethod
    {
        type    gradAlpha;
        delta   cubeRootVol;
    }
    \endverbatim

    Example with psiSurf detection method:
    \verbatim
    interfaceIndicatorMethod
    {
        type    useBlendingMethod;
        blendingMethod
        {
            type                psiSurf;
            critAlpha.air       0.3;
            critAlpha.water     0.3;
            deltaAlpha.air      0.05;
            deltaAlpha.water    0.05;
            f                   0.1;
            n                   5;
        }
    }
    \endverbatim

See also
    Foam::interfaceIndicatorMethods::gradAlpha
    Foam::interfaceIndicatorMethods::useBlendingMethod
    Foam::SGSdelta

SourceFiles
    omegaEgorovRASModel.C

\*---------------------------------------------------------------------------*/

#ifndef omegaEgorovRASModel_H
#define omegaEgorovRASModel_H

#include "phaseSystem.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace RASModels
{

/*---------------------------------------------------------------------------*\
                     Class omegaEgorovRASModel Declaration
\*---------------------------------------------------------------------------*/

template<class BasicMomentumTransportModel>
class omegaEgorovRASModel
:
    public BasicMomentumTransportModel
{
    // Private data

        // Model coefficients

            //- Damping scale
            const dimensionedScalar delta_;


        // Fields

            const phaseModel& phase_;


protected:

        // Protected Member Functions

            virtual tmp<fvScalarMatrix> omegaSource() const;


public:

    typedef typename BasicMomentumTransportModel::alphaField alphaField;
    typedef typename BasicMomentumTransportModel::rhoField rhoField;
    typedef typename BasicMomentumTransportModel::transportModel transportModel;

    //- Runtime type information
    TypeName("omegaEgorovRASModel");


    // Constructors

        //- Construct from components
        omegaEgorovRASModel
        (
            const alphaField& alpha,
            const rhoField& rho,
            const volVectorField& U,
            const surfaceScalarField& alphaRhoPhi,
            const surfaceScalarField& phi,
            const transportModel& transport,
            const word& type = typeName
        );


    //- Destructor
    virtual ~omegaEgorovRASModel()
    {}
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace RASModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
#ifdef NoRepository
    #include "omegaEgorovRASModel.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
