/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2019-2020 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "definitePhases.H"
#include "phaseSystem.H"
#include "addToRunTimeSelectionTable.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace blendingMethods
{
    defineTypeName(definitePhases);

    addToRunTimeSelectionTable
    (
        blendingMethod,
        definitePhases,
        dictionary
    );
}
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::blendingMethods::definitePhases::definitePhases
(
    const dictionary& dict,
    const wordList& phaseNames
)
:
    blendingMethod(dict)
{}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::blendingMethods::definitePhases::~definitePhases()
{}


// * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField> Foam::blendingMethods::definitePhases::f1
(
    const phaseModel& phase1,
    const phaseModel& phase2
) const
{
    if(phase1.fluid().continuousPhases().empty())
    {
        FatalErrorInFunction
            << "The definitePhases blending requires at least one continuous "
            << "phase, specified by keyword 'continuous' in phaseProperties."
            << exit(FatalError);
    }

    const word phases12(IOobject::groupName(phase1.name(), phase2.name()));

    tmp<volScalarField> f1
    (
        volScalarField::New
        (
            IOobject::groupName("f1."+typeName, phases12),
            (!(phase1.continuous()))
           *phase2
           /max(phase2.fluid().sumAlphaContinuous(), phase2.residualAlpha())
        )
    );

    if (debug && phase1.mesh().time().outputTime())
    {
        f1().write();
    }

    return f1;
}


Foam::tmp<Foam::volScalarField> Foam::blendingMethods::definitePhases::f2
(
    const phaseModel& phase1,
    const phaseModel& phase2
) const
{
    if(phase1.fluid().continuousPhases().empty())
    {
        FatalErrorInFunction
        << "The definitePhases blending requires at least one continuous"
        << "phase, specified by keyword 'continuous' in phaseProperties."
            << exit(FatalError);
    }

    const word phases12(IOobject::groupName(phase1.name(), phase2.name()));

    tmp<volScalarField> f2
    (
        volScalarField::New
        (
            IOobject::groupName("f2."+typeName, phases12),
            (!(phase2.continuous()))
           *phase1
           /max(phase1.fluid().sumAlphaContinuous(), phase1.residualAlpha())
        )
    );

    if (debug && phase1.mesh().time().outputTime())
    {
        f2().write();
    }

    return f2;
}


// ************************************************************************* //
