/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2017-2020 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "psiSurf.H"
#include "fvcGrad.H"
#include "addToRunTimeSelectionTable.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace blendingMethods
{
    defineTypeName(psiSurf);

    addToRunTimeSelectionTable
    (
        blendingMethod,
        psiSurf,
        dictionary
    );
}
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::blendingMethods::psiSurf::psiSurf
(
    const dictionary& dict,
    const wordList& phaseNames
)
:
    blendingMethod(dict),

    nCells_
    (
        dict.lookupOrDefault<label>("n", 5)
    ),
    fDelta_
    (
        dict.lookupOrDefault<scalar>("f", 0.1)
    )
{
    forAllConstIter(wordList, phaseNames, iter)
    {
        const word nameCrit
        (
            IOobject::groupName("critAlpha", *iter)
        );

        critAlpha_.insert
        (
            *iter,
            dimensionedScalar
            (
                nameCrit,
                dimless,
                dict.lookup(nameCrit)
            )
        );

        const word nameDelta
        (
            IOobject::groupName("deltaAlpha", *iter)
        );

        deltaAlpha_.insert
        (
            *iter,
            dimensionedScalar
            (
                nameDelta,
                dimless,
                dict.lookup(nameDelta)
            )
        );
    }
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::blendingMethods::psiSurf::~psiSurf()
{}


// * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField> Foam::blendingMethods::psiSurf::f1in2
(
    const phaseModel& dispersed,
    const phaseModel& continuous
) const
{
    const dimensionedScalar crit1(critAlpha_[dispersed.name()]);
    const dimensionedScalar delta1(deltaAlpha_[dispersed.name()]);

    return
        blendingFunction
        (
            limit01
            (
                (dispersed/(dispersed + continuous) - crit1 + delta1)
               /(2*delta1)
            )
        );
}


Foam::tmp<Foam::volScalarField> Foam::blendingMethods::psiSurf::phiSurf
(
    const phaseModel& continuous
) const
{
    static const dimensionedScalar critGrad =
        1/(nCells_*cbrt(average(continuous.mesh().V())));

    static const dimensionedScalar deltaGrad = fDelta_*critGrad;

    return
        blendingFunction
        (
            limit01
            (
                (mag(fvc::grad(continuous)) - critGrad + deltaGrad)
               /(2*deltaGrad)
            )
        );
}


Foam::tmp<Foam::volScalarField> Foam::blendingMethods::psiSurf::f1
(
    const phaseModel& phase1,
    const phaseModel& phase2
) const
{
    const word phases12(IOobject::groupName(phase1.name(), phase2.name()));

    tmp<volScalarField> f1 = phiSurf(phase1)*f1in2(phase1,phase2);

    if (debug && phase1.mesh().time().outputTime())
    {
        f1.ref().rename(IOobject::groupName("f1."+typeName, phases12));
        f1().write();
    }

    return f1;
}


Foam::tmp<Foam::volScalarField> Foam::blendingMethods::psiSurf::f2
(
    const phaseModel& phase1,
    const phaseModel& phase2
) const
{
    const word phases12(IOobject::groupName(phase1.name(), phase2.name()));

    tmp<volScalarField> f2 = phiSurf(phase2)*f1in2(phase2,phase1);

    if (debug && phase1.mesh().time().outputTime())
    {
        f2.ref().rename(IOobject::groupName("f2."+typeName, phases12));
        f2().write();
    }

    return f2;
}


// ************************************************************************* //
