/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2018-2020 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::InducedTurbulencePhaseSystem

Description
    Class which provides infrastructure for induced turbulence.

    Note: This class has to instantiated after momentumTransferPhaseSystem

SourceFiles
    InducedTurbulencePhaseSystem.C

\*---------------------------------------------------------------------------*/

#ifndef InducedTurbulencePhaseSystem_H
#define InducedTurbulencePhaseSystem_H

#include "phaseSystem.H"
#include "HashPtrTable.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

template<class modelType>
class BlendedInterfacialModel;

class blendingMethod;
class inducedTurbulenceModel;

/*---------------------------------------------------------------------------*\
                Class InducedTurbulencePhaseSystem Declaration
\*---------------------------------------------------------------------------*/

template<class BasePhaseSystem>
class InducedTurbulencePhaseSystem
:
    public BasePhaseSystem
{
protected:

    // Protected Typedefs

        typedef HashTable
        <
            autoPtr<BlendedInterfacialModel<inducedTurbulenceModel>>,
            phasePairKey,
            phasePairKey::hash
        > inducedTurbulenceModelTable;

        typedef
        HashPtrTable
        <
            volScalarField,
            phasePairKey,
            phasePairKey::hash
        >
        KkTable;

        typedef HashPtrTable
        <
            volScalarField,
            phasePairKey,
            phasePairKey::hash
        >
        KepsTable;

private:

    // Private Data

        //- Turbulent kinetic energy source terms
        KkTable Kks_;

        //- Turbulent dissipation rate source terms
        KepsTable Kepss_;

        // Sub Models

            //- Induced turbulence models
            inducedTurbulenceModelTable inducedTurbulenceModels_;


public:

    // Constructors

        //- Construct from fvMesh
        InducedTurbulencePhaseSystem(const fvMesh&);


    //- Destructor
    virtual ~InducedTurbulencePhaseSystem();


    // Member Functions

        //- Wrapper to return momentum transfer matrices from
        //  momentumTransferPhaseSystem and to update the tables for the
        //  induced turbulence source terms
        virtual autoPtr<phaseSystem::momentumTransferTable> momentumTransfer();

        //- Turbulent kinetic energy source for phase
        virtual tmp<volScalarField> Kk(const phaseModel& phase) const;

        //- Turbulent kinetic energy source tensor for phase
        virtual tmp<volSymmTensorField> tensorKk
        (
            const phaseModel& phase
        ) const;

        //- Turbulent dissipation rate source for phase
        virtual tmp<volScalarField> Keps(const phaseModel& phase) const;

        //- Read base phaseProperties dictionary
        virtual bool read();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#ifdef NoRepository
    #include "InducedTurbulencePhaseSystem.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
