/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2018-2021 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "ResolvedPhaseSystem.H"

#include "interfaceCompressionModel.H"
#include "surfaceTensionForceModel.H"
#include "subGridSurfaceTensionModel.H"
#include "interfaceIndicatorMethod.H"

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

template<class BasePhaseSystem>
Foam::ResolvedPhaseSystem<BasePhaseSystem>::ResolvedPhaseSystem
(
    const fvMesh& mesh
)
:
    BasePhaseSystem(mesh)
{
    this->generatePairsAndSubModels
    (
        "interfaceCompression",
        interfaceCompressionModels_
    );

    if (this->found("subGridSurfaceTension"))
    {
        this->generatePairsAndSubModels
        (
            "subGridSurfaceTension",
            subGridSurfaceTensionModels_
        );
    }

    surfaceTensionForceModel_ = surfaceTensionForceModel::New(*this);

    interfaceIndicatorMethod_ = interfaceIndicatorMethod::New(*this);
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

template<class BasePhaseSystem>
Foam::ResolvedPhaseSystem<BasePhaseSystem>::~ResolvedPhaseSystem()
{}


// * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * * //

template<class BasePhaseSystem>
Foam::tmp<Foam::surfaceScalarField>
Foam::ResolvedPhaseSystem<BasePhaseSystem>::phir
(
    const phaseModel& phase1,
    const phaseModel& phase2
) const
{
    tmp<surfaceScalarField> phir(BasePhaseSystem::phir(phase1, phase2));

    phasePairKey key(phase1.name(), phase2.name());

    if (interfaceCompressionModels_.found(key))
    {
        if (interfaceCompressionModels_[key]().type() != "none")
        {
            phir.ref() += interfaceCompressionModels_[key]().phirc(phase1);
        }
    }

    return phir;
}


template<class BasePhaseSystem>
void Foam::ResolvedPhaseSystem<BasePhaseSystem>::addCompressionFlux
(
    const phaseModel& phase,
    surfaceScalarField& alphaPhi
) const
{
    BasePhaseSystem::addCompressionFlux(phase, alphaPhi);

    if (!interfaceCompressionModels_.empty())
    {
        const volScalarField& alpha = phase;

        forAll(this->phases(), phasei)
        {
            const phaseModel& phase2 = this->phases()[phasei];
            const volScalarField& alpha2 = phase2;
            const phasePairKey key(phase.name(), phase2.name());

            if (&phase2 == &phase) continue;

            interfaceCompressionModelTable::const_iterator cAlpha
            (
                interfaceCompressionModels_.find(key)
            );

            if (cAlpha != interfaceCompressionModels_.end())
            {
                const surfaceScalarField phir
                (
                    interfaceCompressionModels_[key]().phirc(phase)
                );

                const word phirScheme
                (
                    "div(phir,"
                  + alpha2.name() + ',' + alpha.name()
                  + ')'
                );

                alphaPhi += fvc::flux
                (
                    -fvc::flux(-phir, alpha2, phirScheme),
                    alpha,
                    phirScheme
                );
            }
        }
    }
}


template<class BasePhaseSystem>
Foam::tmp<Foam::surfaceScalarField>
Foam::ResolvedPhaseSystem<BasePhaseSystem>::surfaceTension
(
    const phaseModel& phase1
) const
{
    tmp<surfaceScalarField> tSurfaceTension
    (
        new surfaceScalarField
        (
            IOobject
            (
                "surfaceTension",
                this->mesh().time().timeName(),
                this->mesh()
            ),
            this->mesh(),
            dimensionedScalar
            (
                "surfaceTension",
                dimensionSet(1, -2, -2, 0, 0),
                0
            )
        )
    );

    forAll(this->phases(), phasej)
    {
        const phaseModel& phase2 = this->phases()[phasej];

        if (&phase2 != &phase1)
        {
            phasePairKey key12(phase1.name(), phase2.name());

            if (interfaceCompressionModels_.found(key12))
            {
                if (interfaceCompressionModels_[key12]().type() != "none")
                {
                    const phasePair& pair(this->phasePairs_[key12]);

                    tSurfaceTension.ref() +=
                        surfaceTensionForceModel_->surfaceTensionForcef(pair);

                    if (subGridSurfaceTensionModels_.found(key12))
                    {
                        if
                        (
                            subGridSurfaceTensionModels_[key12]().type()
                         != "none"
                        )
                        {
                            tSurfaceTension.ref() +=
                                subGridSurfaceTensionModels_[key12]().
                                subGridSurfaceTensionForcef
                                (
                                    surfaceTensionForceModel_->K(pair)
                                );
                        }
                    }
                }
            }
        }
    }

    return tSurfaceTension;
}


template<class BasePhaseSystem>
Foam::tmp<Foam::volScalarField>
Foam::ResolvedPhaseSystem<BasePhaseSystem>::interfaceIndicator
(
    const phaseModel& phase
) const
{
   return interfaceIndicatorMethod_->interfaceIndicator(phase);
}


template<class BasePhaseSystem>
bool Foam::ResolvedPhaseSystem<BasePhaseSystem>::read()
{
    if (BasePhaseSystem::read())
    {
        bool readOK = true;

        // Read models ...

        return readOK;
    }
    else
    {
        return false;
    }
}


// ************************************************************************* //
