/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2018-2020 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::interfaceCompressionModel

Description
    Base class for modeling the interface compression flux which is
    added to the correction fluxes within multiphaseSystem::solveAlphas().

SourceFiles
    interfaceCompressionModel.C
    interfaceCompressionModelNew.C
    phaseSystem.C

\*---------------------------------------------------------------------------*/

#ifndef interfaceCompressionModel_H
#define interfaceCompressionModel_H

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#include "volFields.H"
#include "dictionary.H"
#include "runTimeSelectionTables.H"

namespace Foam
{

class phasePair;
class phaseModel;

/*---------------------------------------------------------------------------*\
                  Class interfaceCompressionModel Declaration
\*---------------------------------------------------------------------------*/

class interfaceCompressionModel
{
protected:

    // Protected data

        //- Phase pair
        const phasePair& pair_;

        //- Reference to the mesh
        const fvMesh& mesh_;

        //- Stabilisation for normalisation of the interface normal
        const dimensionedScalar deltaN_;


public:

    //- Runtime type information
    TypeName("interfaceCompressionModel");


    // Declare runtime construction

        declareRunTimeSelectionTable
        (
            autoPtr,
            interfaceCompressionModel,
            dictionary,
            (
                const dictionary& dict,
                const phasePair& pair
            ),
            (dict, pair)
        );


    // Constructors

        //- Construct from a dictionary and a phase pair
        interfaceCompressionModel
        (
            const dictionary& dict,
            const phasePair& pair
        );


    //- Destructor
    virtual ~interfaceCompressionModel();


    // Selectors

        static autoPtr<interfaceCompressionModel> New
        (
            const dictionary& dict,
            const phasePair& pair
        );


    // Member Functions

        //- Interface compression coefficient
        virtual tmp<surfaceScalarField> C() const = 0;

        //- Face unit interface normal flux
        virtual tmp<surfaceScalarField> nHatf(const phaseModel& phase) const;

        //- Interface compression flux
        virtual tmp<surfaceScalarField> phirc(const phaseModel& phase) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
