/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2020-2021 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "addToRunTimeSelectionTable.H"
#include "fvcGrad.H"
#include "interfaceIndicatorMethod.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
    defineTypeNameAndDebug(interfaceIndicatorMethod, 0);
    defineRunTimeSelectionTable(interfaceIndicatorMethod, dictionary);
}


// * * * * * * * * * * * * Protected Member Functions  * * * * * * * * * * * //

//- Provides default implementation for pure virtual function
Foam::tmp<Foam::volScalarField>
Foam::interfaceIndicatorMethod::interfaceIndicator
(
    const phaseModel& phase1,
    const phaseModel& phase2
) const
{
    NotImplemented;
    return tmp<volScalarField>(nullptr);
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::interfaceIndicatorMethod::interfaceIndicatorMethod
(
    const phaseSystem& fluid
)
:
    fluid_(fluid)
{}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::interfaceIndicatorMethod::~interfaceIndicatorMethod()
{}


// * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::interfaceIndicatorMethod::interfaceIndicator
(
    const phaseModel& phase
) const
{
    // Consider only interfaces between continuous phases
    if (!phase.continuous())
    {
        FatalErrorInFunction
            << "Attempt to detect interface for non-continuous phase: "
            << phase.name() << endl
            << exit(FatalError);
    }
    const phaseSystem::phaseModelPartialList& phases(fluid_.continuousPhases());

    tmp<volScalarField> ri
    (
        new volScalarField
        (
            IOobject
            (
                IOobject::groupName("interfaceIndicator", phase.name()),
                fluid_.mesh().time().timeName(),
                fluid_.mesh()
            ),
            fluid_.mesh(),
            dimensionedScalar(dimless, Zero)
        )
    );

    // Use maximum pair contribution
    forAll(phases, phasei)
    {
        const phaseModel& phase2 = phases[phasei];

        if (phase2 != phase)
        {
            ri.ref() = max(this->interfaceIndicator(phase, phase2), ri());
        }
    }

    if (debug && fluid_.mesh().time().outputTime()) ri().write();

    return ri;
}


// ************************************************************************* //
