/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2020-2021 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::interfaceIndicatorMethod

Description
    Base class for methods to track resolved interfaces in a multiphase system.

    Determine the interface indicator field for a continuous phase i
    by determining the maximum contribution from interface pairs between
    phases i and j:

    \f[
        \mathrm{RI}_i = \mathrm{max}(\mathrm{RI}_{ij})\text{, where }i \neq j.
    \f]

SourceFiles
    interfaceIndicatorMethod.C

\*---------------------------------------------------------------------------*/

#ifndef interfaceIndicatorMethod_H
#define interfaceIndicatorMethod_H

#include "dictionary.H"
#include "runTimeSelectionTables.H"
#include "phaseModel.H"
#include "phaseSystem.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                   Class interfaceIndicatorMethod Declaration
\*---------------------------------------------------------------------------*/

class interfaceIndicatorMethod
{
protected:

    // Protected References

        //- Reference to the phaseSystem
        const phaseSystem& fluid_;

    // Protected Member Functions

        //- Indicator field between a phase pair
        virtual tmp<volScalarField> interfaceIndicator
        (
            const phaseModel& phase1,
            const phaseModel& phase2
        ) const = 0;


public:

    //- Runtime type information
    TypeName("interfaceIndicatorMethod");


    // Declare runtime construction
    declareRunTimeSelectionTable
    (
        autoPtr,
        interfaceIndicatorMethod,
        dictionary,
        (
            const phaseSystem& fluid
        ),
        (fluid)
    );


    // Constructors

        //- Construct from a dictionary from phaseSystem
        interfaceIndicatorMethod
        (
            const phaseSystem& fluid
        );


    // Selector

        static autoPtr<interfaceIndicatorMethod> New
        (
            const phaseSystem& fluid
        );


    //- Destructor
    virtual ~interfaceIndicatorMethod();


    // Member Functions

        //- Indicator field of the proximity of a resolved phase interface
        //  Values are distributed from 0 (far) to 1 (near).
        virtual tmp<volScalarField> interfaceIndicator
        (
            const phaseModel& phase
        ) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
