/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2020 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::interfaceIndicatorMethods::useBlendingMethod

Description
    Determine the interface indicator field for a pair of continuous phases,
    i and j:

    \f[
        \mathrm{RI}_{ij} = (1 - f_1 - f_2)_{ij},
    \f]

    where \f$f_1\f$ and \f$f_2\f$ are provided by the selected blendingMethod.

Usage
    Example setup for continuous air and water phases:
    \verbatim
    interfaceIndicatorMethod
    {
        type    useBlendingMethod;
        blendingMethod
        {
            type                psiSurf;
            critAlpha.air       0.3;
            critAlpha.water     0.3;
            deltaAlpha.air      0.05;
            deltaAlpha.water    0.05;
            f                   0.1;
            n                   5;
        }
    }
    \endverbatim

See also
    Foam::blendingMethod
    Foam::blendingMethods::psiSurf

SourceFiles
    useBlendingMethod.C

\*---------------------------------------------------------------------------*/

#ifndef useBlendingMethod_H
#define useBlendingMethod_H

#include "blendingMethod.H"
#include "interfaceIndicatorMethod.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace interfaceIndicatorMethods
{

/*---------------------------------------------------------------------------*\
                      Class useBlendingMethod Declaration
\*---------------------------------------------------------------------------*/

class useBlendingMethod
:
    public interfaceIndicatorMethod
{
private:

    // Private Data

        //- Blending method used for interface detection
        autoPtr<blendingMethod> blendingMethod_;


protected:

    // Protected Member Functions

        //- Indicator field between a phase pair
        virtual tmp<volScalarField> interfaceIndicator
        (
            const phaseModel& phase1, const phaseModel& phase2
        ) const override;


public:

    //- Runtime type information
    TypeName("useBlendingMethod");


    // Constructors

        //- Construct from a dictionary from phaseSystem
        useBlendingMethod
        (
            const phaseSystem& fluid
        );


    //- Destructor
    virtual ~useBlendingMethod();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace interfaceIndicatorMethods
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
