/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2021 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "continuumSurfaceForce.H"
#include "phasePair.H"
#include "addToRunTimeSelectionTable.H"
#include "alphaContactAngleFvPatchScalarField.H"
#include "unitConversion.H"
#include "fvc.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace surfaceTensionForceModels
{
    defineTypeNameAndDebug(continuumSurfaceForce, 0);
    addToRunTimeSelectionTable
    (
        surfaceTensionForceModel,
        continuumSurfaceForce,
        dictionary
    );
}
}


// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //

void
Foam::surfaceTensionForceModels::continuumSurfaceForce::correctContactAngle
(
    surfaceVectorField::Boundary& nHatb,
    const phasePair& pair
) const
{
    // Correction for the boundary condition on the unit normal nHat on
    // walls to produce the correct contact angle.

    // The dynamic contact angle is calculated from the component of the
    // velocity on the direction of the interface, parallel to the wall.

    const volScalarField::Boundary& gbf = pair.phase1().boundaryField();

    const fvBoundaryMesh& boundary = this->fluid_.mesh().boundary();

    forAll(boundary, patchi)
    {
        if (isA<alphaContactAngleFvPatchScalarField>(gbf[patchi]))
        {
            const alphaContactAngleFvPatchScalarField& acap =
                refCast<const alphaContactAngleFvPatchScalarField>(gbf[patchi]);

            vectorField& nHatPatch = nHatb[patchi];

            vectorField AfHatPatch
            (
                this->fluid_.mesh().Sf().boundaryField()[patchi]
               /this->fluid_.mesh().magSf().boundaryField()[patchi]
            );

            alphaContactAngleFvPatchScalarField::thetaPropsTable::
                const_iterator tp =
                    acap.thetaProps()
                   .find
                    (
                        phasePairKey
                        (
                            pair.phase1().name(),
                            pair.phase2().name()
                        )
                    );

            if (tp == acap.thetaProps().end())
            {
                FatalErrorInFunction
                    << "Cannot find interface "
                    << phasePairKey
                       (
                            pair.phase1().name(),
                            pair.phase2().name()
                       )
                    << "\n    in table of theta properties for patch "
                    << acap.patch().name()
                    << exit(FatalError);
            }

            bool matched = (tp.key().first() == pair.phase1().name());

            scalar theta0 = degToRad(tp().theta0(matched));
            scalarField theta(boundary[patchi].size(), theta0);

            scalar uTheta = tp().uTheta();

            // Calculate the dynamic contact angle if required
            if (uTheta > small)
            {
                scalar thetaA = degToRad(tp().thetaA(matched));
                scalar thetaR = degToRad(tp().thetaR(matched));

                // Calculated the component of the velocity parallel to the wall
                vectorField Uwall
                (
                    pair.phase1().U()().boundaryField()[patchi].
                    patchInternalField()
                  - pair.phase1().U()().boundaryField()[patchi]
                );
                Uwall -= (AfHatPatch & Uwall)*AfHatPatch;

                // Find the direction of the interface parallel to the wall
                vectorField nWall
                (
                    nHatPatch - (AfHatPatch & nHatPatch)*AfHatPatch
                );

                // Normalise nWall
                nWall /= (mag(nWall) + small);

                // Calculate Uwall resolved normal to the interface parallel to
                // the interface
                scalarField uwall(nWall & Uwall);

                theta += (thetaA - thetaR)*tanh(uwall/uTheta);
            }


            // Reset nHatPatch to correspond to the contact angle

            scalarField a12(nHatPatch & AfHatPatch);

            scalarField b1(cos(theta));

            scalarField b2(nHatPatch.size());

            forAll(b2, facei)
            {
                b2[facei] = cos(acos(a12[facei]) - theta[facei]);
            }

            scalarField det(1 - a12*a12);

            scalarField a((b1 - a12*b2)/det);
            scalarField b((b2 - a12*b1)/det);

            nHatPatch = a*AfHatPatch + b*nHatPatch;

            nHatPatch /= (mag(nHatPatch) + deltaN_.value());
        }
    }
}


// * * * * * * * * * * * * Protected Member Functions  * * * * * * * * * * * //

Foam::tmp<Foam::surfaceVectorField>
Foam::surfaceTensionForceModels::continuumSurfaceForce::nHatfv
(
    const phasePair& pair
) const
{
    surfaceVectorField gradAlphafv
    (
        fvc::interpolate(pair.phase2())
       *fvc::interpolate(fvc::grad(pair.phase1()))
      - fvc::interpolate(pair.phase1())
       *fvc::interpolate(fvc::grad(pair.phase2()))
    );

    return gradAlphafv/(mag(gradAlphafv) + deltaN_);
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::surfaceTensionForceModels::continuumSurfaceForce::continuumSurfaceForce
(
    const phaseSystem& fluid
)
:
    surfaceTensionForceModel(fluid),
    deltaN_
    (
        "deltaN",
        1e-8/pow(average(this->fluid_.mesh().V()), 1.0/3.0)
    )
{}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::surfaceTensionForceModels::continuumSurfaceForce::~continuumSurfaceForce()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::surfaceTensionForceModels::continuumSurfaceForce::K
(
    const phasePair& pair
) const
{
    tmp<surfaceVectorField> tnHatfv = this->nHatfv(pair);

    correctContactAngle(tnHatfv.ref().boundaryFieldRef(), pair);

    return -fvc::div(tnHatfv & this->fluid_.mesh().Sf());
}


// ************************************************************************* //
