/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2021 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::surfaceTensionForceModels::smoothedContinuumSurfaceForce

Description
    Smoothing alpha fields prior to calculating the gradient vector of a phase
    pair to get interface curvature helps to dampen spurious currents. A number
    of two smoothing iterations show the least spurious currents.

    Reference:
    \verbatim
        Ubbink, O. (1997).
        Numerical prediction of two fluid systems with sharp interfaces.
        PhD Thesis, 193 pp. Imperial College of Science, Technology & Medicine
        London.
    \endverbatim

Usage
    \table
        Name         | Description                        | Default
        nSmoothIters | Number of iterations for smoothing | 2
    \endtable

SourceFiles
    smoothedContinuumSurfaceForce.C

\*---------------------------------------------------------------------------*/

#ifndef smoothedContinuumSurfaceForce_H
#define smoothedContinuumSurfaceForce_H

#include "continuumSurfaceForce.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

class phasePair;

namespace surfaceTensionForceModels
{

/*---------------------------------------------------------------------------*\
                    Class smoothedContinuumSurfaceForce Declaration
\*---------------------------------------------------------------------------*/

class smoothedContinuumSurfaceForce
:
    public continuumSurfaceForce
{
    // Private Data

        //- Number of times to smoothen field
        const label nSmoothIters_;


    // Private Member Functions

        //- Smoothen a volScalarField for defined number of times
        tmp<volScalarField> smoothen(const volScalarField& vsf) const;


protected:

    // Protected Member Functions

        //- Return the face-based interface normal vector
        virtual tmp<surfaceVectorField> nHatfv(const phasePair& pair) const;


public:

    //- Runtime type information
    TypeName("smoothedContinuumSurfaceForce");


    // Constructors

        //- Construct from a dictionary and a phase pair
        smoothedContinuumSurfaceForce(const phaseSystem& fluid);


    //- Destructor
    virtual ~smoothedContinuumSurfaceForce();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace surfaceTensionForceModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
