/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2019-2021 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "Liao.H"
#include "addToRunTimeSelectionTable.H"
#include "phaseCompressibleMomentumTransportModel.H"
#include "mathematicalConstants.H"
#include "fvcGrad.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace diameterModels
{
namespace binaryBreakupModels
{
    defineTypeNameAndDebug(Liao, 0);
    addToRunTimeSelectionTable
    (
        binaryBreakupModel,
        Liao,
        dictionary
    );
}
}
}

using Foam::constant::mathematical::pi;


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::diameterModels::binaryBreakupModels::Liao::Liao
(
    const populationBalanceModel& popBal,
    const dictionary& dict
)
:
    binaryBreakupModel(popBal, dict),
    BTurb_(dimensionedScalar::lookupOrDefault("BTurb", dict, dimless, 1)),
    BShear_(dimensionedScalar::lookupOrDefault("BShear", dict, dimless, 1)),
    BEddy_(dimensionedScalar::lookupOrDefault("BEddy", dict, dimless, 1)),
    BFric_(dimensionedScalar::lookupOrDefault("BFric", dict, dimless, 0.25)),
    turbulence_(dict.lookup("turbulence")),
    laminarShear_(dict.lookup("laminarShear")),
    turbulentShear_(dict.lookup("turbulentShear")),
    interfacialFriction_(dict.lookup("interfacialFriction")),
    kolmogorovLengthScale_
    (
        IOobject
        (
            "kolmogorovLengthScale",
            popBal_.time().timeName(),
            popBal_.mesh()
        ),
        popBal_.mesh(),
        dimensionedScalar
        (
            "kolmogorovLengthScale",
            dimLength,
            Zero
        )
    ),
    shearStrainRate_
    (
        IOobject
        (
            "shearStrainRate",
            popBal_.time().timeName(),
            popBal_.mesh()
        ),
        popBal_.mesh(),
        dimensionedScalar
        (
            "shearStrainRate",
            dimVelocity/dimLength,
            Zero
        )
    ),
    eddyStrainRate_
    (
        IOobject
        (
            "eddyStrainRate",
            popBal_.time().timeName(),
            popBal_.mesh()
        ),
        popBal_.mesh(),
        dimensionedScalar
        (
            "eddyStrainRate",
            dimVelocity/dimLength,
            Zero
        )
    )

{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::diameterModels::binaryBreakupModels::Liao::precompute()
{
    kolmogorovLengthScale_ =
        pow025
        (
            pow3
            (
                popBal_.continuousPhase().thermo().nu()
            )
           /popBal_.continuousTurbulence().epsilon()
        );

    shearStrainRate_ =
        sqrt(2.0)
       *mag(symm(fvc::grad(popBal_.continuousPhase().U())));

    // Eq. (23)
    eddyStrainRate_ = sqrt
           (
               popBal_.continuousPhase().rho()
              *popBal_.continuousTurbulence().epsilon()
              /popBal_.continuousPhase().thermo().mu()
           );

    if (uTerminal_.empty())
    {
        // Calculate terminal velocity and drag coefficient
        // using the IshiiZuberSparse model

        const uniformDimensionedVectorField& g =
            popBal_.mesh().lookupObject<uniformDimensionedVectorField>("g");

        const dimensionedScalar nuc
        (
            "nuc",
            dimViscosity,
            gAverage(popBal_.continuousPhase().thermo().nu()())
        );

        const dimensionedScalar rhoc
        (
            "rhoc", dimDensity, gAverage(popBal_.continuousPhase().rho()())
        );

        const dimensionedScalar rhod
        (
            "rhod",
            dimDensity,
            gAverage(popBal_.sizeGroups()[1].phase().rho()())
        );

        const dimensionedScalar sigma
        (
            "sigma",
            dimForce/dimLength,
            gAverage
            (
                popBal_.sigmaWithContinuousPhase
                (
                    popBal_.sizeGroups()[1].phase()
                )()
            )
        );

        for(int m = 0; m < popBal_.sizeGroups().size(); m++)
        {
            const sizeGroup& f = popBal_.sizeGroups()[m];

            // Initial values
            dimensionedScalar uTerminal("uTerminal", dimVelocity, 0.2);
            dimensionedScalar Cd("Cd", dimless, 0.44);
            dimensionedScalar CdEllipse("CdEllipse", dimless, 1);

            dimensionedScalar Re(uTerminal*f.dSph()/nuc);
            dimensionedScalar Eo(mag(g)*mag(rhoc - rhod)*sqr(f.dSph())/sigma);

            // Scalars for the iteration
            dimensionedScalar F("F", dimForce/dimArea, 1);
            dimensionedScalar dF("dF", dimForce/dimArea/dimVelocity, 1);
            dimensionedScalar uTerminalX("uTerminalX", dimVelocity, 1.0e-05);
            dimensionedScalar ReX("ReX", dimless, Re.value());
            dimensionedScalar CdX("CdX", dimless, Cd.value());
            dimensionedScalar dCd("dCd", Cd.dimensions()/dimVelocity, Zero);

            int n = 0;

            while(mag(F.value()) >= 1.0e-05 && n++ <= 20)
            {
                Re = uTerminal*f.dSph()/nuc;

                Cd =
                    pos0(1000 - Re)*24/Re*(1 + 0.1*pow(Re, 0.75))
                  + neg(1000 - Re)*0.44;

                CdEllipse = 0.6666*sqrt(Eo);

                Cd =
                    pos0(CdEllipse - Cd)
                   *min(CdEllipse.value(), 2.66667)
                  + neg(CdEllipse - Cd)*Cd;

                F =
                    4.0/3.0*(rhoc - rhod)*mag(g)*f.dSph()
                  - rhoc*Cd*sqr(uTerminal);

                ReX = (uTerminal + uTerminalX)*f.dSph()/nuc;
                CdX =
                    pos0(1000 - ReX)
                   *24/ReX*(1 + 0.1*pow(ReX, 0.75))
                  + neg(1000 - ReX)*0.44;

                CdX =
                    pos0(CdEllipse - CdX)
                   *min(CdEllipse.value(), 2.66667)
                  + neg(CdEllipse - CdX)*CdX;

                dCd = (CdX - Cd)/uTerminalX;

                dF = -(2*rhoc*uTerminal*Cd + rhoc*sqr(uTerminal)*dCd);

                uTerminal -= F/dF;
            }

            uTerminal_.append(new dimensionedScalar("uTerminal", uTerminal));

            Cd_.append(new dimensionedScalar("Cd", Cd));
        }
    }
}


void Foam::diameterModels::binaryBreakupModels::Liao::
addToBinaryBreakupRate
(
    volScalarField& binaryBreakupRate,
    const label i,
    const label j
)
{
    const phaseModel& continuousPhase = popBal_.continuousPhase();
    const sizeGroup& fi = popBal_.sizeGroups()[i];
    const sizeGroup& fj = popBal_.sizeGroups()[j];

    dimensionedScalar dk(cbrt(pow3(fj.dSph()) - pow3(fi.dSph())));

    // Eq. (34)
    const volScalarField tauCrit1
    (
        6*popBal_.sigmaWithContinuousPhase(fj.phase())/fj.dSph()
       *(sqr(fi.dSph()/fj.dSph()) + sqr(dk/fj.dSph()) - 1)
    );

    // Eq. (35)
    const volScalarField tauCrit2
    (
        popBal_.sigmaWithContinuousPhase(fj.phase())/min(dk, fi.dSph())
    );

    // Eq. (33)
    const volScalarField tauCrit(max(tauCrit1, tauCrit2));

    // Eqs. (32) and (36)
    if (turbulence_)
    {
        const volScalarField tauTurb
        (
            pos(fj.dSph() - kolmogorovLengthScale_)*BTurb_*continuousPhase.rho()
           *sqr(cbrt(popBal_.continuousTurbulence().epsilon()*fj.dSph()))
        );

        binaryBreakupRate +=
            pos(tauTurb - tauCrit)*1/fj.dSph()
           *sqrt(mag(tauTurb - tauCrit)/continuousPhase.rho())/fj.x();
    }

    if (laminarShear_)
    {
        const volScalarField tauShear
        (
            BShear_*continuousPhase.thermo().mu()*shearStrainRate_
        );

        binaryBreakupRate +=
            pos(tauShear - tauCrit)*1/fj.dSph()
           *sqrt(mag(tauShear - tauCrit)/continuousPhase.rho())/fj.x();
    }

    if (turbulentShear_)
    {
        const volScalarField tauEddy
        (
            pos0(kolmogorovLengthScale_ - fj.dSph())
           *BEddy_*continuousPhase.thermo().mu()*eddyStrainRate_
        );

        binaryBreakupRate +=
            pos(tauEddy - tauCrit)*1/fj.dSph()
           *sqrt(mag(tauEddy - tauCrit)/continuousPhase.rho())/fj.x();
    }

    if (interfacialFriction_)
    {
        const volScalarField tauFric
        (
            BFric_*0.5*continuousPhase.rho()*sqr(uTerminal_[j])*Cd_[j]
        );

        binaryBreakupRate +=
            pos(tauFric - tauCrit)*1/fj.dSph()
           *sqrt(mag(tauFric - tauCrit)/continuousPhase.rho())/fj.x();
    }
}


// ************************************************************************* //
