/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2019-2020 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::diameterModels::binaryBreakupModels::Liao

Description
    Model of Liao et al. (2015).

    Reference:
    \verbatim
        Liao, Y.; Rzehak, R.; Lucas, D.; Krepper, E. (2015).
        Baseline closure models for dispersed bubbly flow:
        Bubble coalescence and breakup.
        Chemical Engineering Science, 122, 336-349.
    \endverbatim

    Note that compared to the reference above indices i and j for the mother
    and daughter bubble are interchanged here.
    Note also that in order to include the model into the volume-based class
    method the breakup rate is divided by the volume of the mother bubble.
    In the current version constant fluid densities are assumed.
    For each size group the terminal velocity and the drag coefficient are
    computed once iteratively using the Newton-Raphson method with the
    IshiiZuberSparse drag correlation.

Usage
    \table
        Property       | Description              | Required    | Default value
        BTurb          | coefficient BTurb        | no          | 1.0
        BShear         | coefficient Bbuoy        | no          | 1.0
        BEddy          | coefficient BEddy        | no          | 1.0
        BFric          | coefficient Bwake        | no          | 1.0
        turbulence     | Switch for turbulence    | yes         | none
        laminarShear   | Switch for laminar shear | yes         | none
        turbulentShear | Switch for eddies        | yes         | none
        friction       | Switch for friction      | yes         | none
    \endtable

SourceFiles
    Liao.C

\*---------------------------------------------------------------------------*/

#ifndef Liao_H
#define Liao_H

#include "binaryBreakupModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace diameterModels
{
namespace binaryBreakupModels
{

/*---------------------------------------------------------------------------*\
                            Class Liao Declaration
\*---------------------------------------------------------------------------*/

class Liao
:
    public binaryBreakupModel
{
    // Private Data

        //- Optional coefficient BTurb
        dimensionedScalar BTurb_;

        //- Optional coefficient BShear
        dimensionedScalar BShear_;

        //- Optional coefficient BEddy
        dimensionedScalar BEddy_;

        //- Optional coefficient BFric
        dimensionedScalar BFric_;

        //- Switch for considering turbulent velocity fluctuation
        Switch turbulence_;

        //- Switch for considering velocity gradients in the bulk
        Switch laminarShear_;

        //- Switch for considering eddies
        Switch turbulentShear_;

        //- Switch for considering interfacial friction
        Switch interfacialFriction_;

        //- Kolmogorov length scale
        volScalarField kolmogorovLengthScale_;

        //- Shear strain rate
        volScalarField shearStrainRate_;

        //- Eddy strain rate
        volScalarField eddyStrainRate_;

        //- Terminal velocities
        PtrList<dimensionedScalar> uTerminal_;

        //- Drag coefficients
        PtrList<dimensionedScalar> Cd_;


public:

    //- Runtime type information
    TypeName("Liao");

    // Constructor

        Liao
        (
            const populationBalanceModel& popBal,
            const dictionary& dict
        );


    //- Destructor
    virtual ~Liao()
    {}


    // Member Functions

        //- Precompute diameter independent expressions
        virtual void precompute();

        //- Add to binary breakupRate
        virtual void addToBinaryBreakupRate
        (
            volScalarField& binaryBreakupRate,
            const label i,
            const label j
        );
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace binaryBreakupModels
} // End namespace diameterModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
