/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2019-2021 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "LiaoCoalescence.H"
#include "addToRunTimeSelectionTable.H"
#include "mathematicalConstants.H"
#include "phaseCompressibleMomentumTransportModel.H"
#include "fvcGrad.H"
#include "dragModel.H"
#include "phaseSystem.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace diameterModels
{
namespace coalescenceModels
{
    defineTypeNameAndDebug(LiaoCoalescence, 0);
    addToRunTimeSelectionTable
    (
        coalescenceModel,
        LiaoCoalescence,
        dictionary
    );
}
}
}

using Foam::constant::mathematical::pi;


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::diameterModels::coalescenceModels::LiaoCoalescence::
LiaoCoalescence
(
    const populationBalanceModel& popBal,
    const dictionary& dict
)
:
    coalescenceModel(popBal, dict),
    PMax_(dimensionedScalar::lookupOrDefault("PMax", dict, dimless, 0.8)),
    AH_(dimensionedScalar::lookupOrDefault("AH", dict, dimEnergy, 3.7e-20)),
    CEff_(dimensionedScalar::lookupOrDefault("CEff", dict, dimless, 5)),
    CTurb_(dimensionedScalar::lookupOrDefault("CTurb", dict, dimless, 1)),
    CBuoy_(dimensionedScalar::lookupOrDefault("CBuoy", dict, dimless, 1)),
    CShear_(dimensionedScalar::lookupOrDefault("CShear", dict, dimless, 1)),
    CEddy_(dimensionedScalar::lookupOrDefault("CEddy", dict, dimless, 1)),
    CWake_(dimensionedScalar::lookupOrDefault("CWake", dict, dimless, 1)),
    turbulence_(dict.lookup("turbulence")),
    buoyancy_(dict.lookup("buoyancy")),
    laminarShear_(dict.lookup("laminarShear")),
    eddyCapture_(dict.lookup("eddyCapture")),
    wakeEntrainment_(dict.lookup("wakeEntrainment")),
    kolmogorovLengthScale_
    (
        IOobject
        (
            "kolmogorovLengthScale",
            popBal_.time().timeName(),
            popBal_.mesh()
        ),
        popBal_.mesh(),
        dimensionedScalar
        (
            "kolmogorovLengthScale",
            dimLength,
            Zero
        )
    ),
    CPack_
    (
        IOobject
        (
            "CPack",
            popBal_.time().timeName(),
            popBal_.mesh()
        ),
        popBal_.mesh(),
        dimensionedScalar
        (
            "CPack",
            dimless,
            Zero
        )
    ),
    CPackMax_
    (
        dimensionedScalar::lookupOrDefault("CPackMax", dict, dimless, 1e5)
    ),
    shearStrainRate_
    (
        IOobject
        (
            "shearStrainRate",
            popBal_.time().timeName(),
            popBal_.mesh()
        ),
        popBal_.mesh(),
        dimensionedScalar
        (
            "shearStrainRate",
            dimVelocity/dimLength,
            Zero
        )
    ),
    eddyStrainRate_
    (
        IOobject
        (
            "eddyStrainRate",
            popBal_.time().timeName(),
            popBal_.mesh()
        ),
        popBal_.mesh(),
        dimensionedScalar
        (
            "eddyStrainRate",
            dimVelocity/dimLength,
            Zero
        )
    ),
    dCrit_
    (
        IOobject
        (
            "dCrit",
            popBal_.time().timeName(),
            popBal_.mesh()
        ),
        popBal_.mesh(),
        dimensionedScalar
        (
            "dCrit",
            dimLength,
            Zero
        )
    ),
    uRelTurb_
    (
        IOobject
        (
            "uRelTurb",
            popBal_.time().timeName(),
            popBal_.mesh()
        ),
        popBal_.mesh(),
        dimensionedScalar
        (
            "uRelTurb",
            dimVelocity,
            Zero
        )
    ),
    uRelBuoy_
    (
        IOobject
        (
            "uRelBuoy",
            popBal_.time().timeName(),
            popBal_.mesh()
        ),
        popBal_.mesh(),
        dimensionedScalar
        (
            "uRelBuoy",
            dimVelocity,
            Zero
        )
    ),
    uRelShear_
    (
        IOobject
        (
            "uRelShear",
            popBal_.time().timeName(),
            popBal_.mesh()
        ),
        popBal_.mesh(),
        dimensionedScalar
        (
            "uRelShear",
            dimVelocity,
            Zero
        )
    )

{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::diameterModels::coalescenceModels::LiaoCoalescence::precompute()
{
    if (PMax_.value() <= gMax(popBal_.alphas()))
    {
        Warning
            << "Gas volume fraction exceeds the maximum packing limit" <<endl;
    }

    CPack_ = min(PMax_/max(PMax_ - popBal_.alphas(), SMALL), CPackMax_);

    kolmogorovLengthScale_ =
        pow025
        (
            pow3
            (
                popBal_.continuousPhase().thermo().nu()
            )
           /popBal_.continuousTurbulence().epsilon()
        );

    shearStrainRate_ =
        sqrt(2.0)
       *mag(symm(fvc::grad(popBal_.continuousPhase().U())));

    // Eq. (23)
    eddyStrainRate_ =
        sqrt
        (
            popBal_.continuousPhase().rho()
           *popBal_.continuousTurbulence().epsilon()
           /popBal_.continuousPhase().thermo().mu()
        );

    const uniformDimensionedVectorField& g =
        popBal_.mesh().lookupObject<uniformDimensionedVectorField>("g");

    // Eq. (27)
    dCrit_ =
        4*sqrt
        (
            popBal_.sigmaWithContinuousPhase(popBal_.sizeGroups()[1].phase())()
           /(mag(g)*(popBal_.continuousPhase().rho()
          - popBal_.sizeGroups()[1].phase().rho()()))
        );

    if (uTerminal_.empty())
    {
        // Calculate terminal velocity and drag coefficient
        // using the IshiiZuberSparse model

        const dimensionedScalar nuc
        (
            "nuc",
            dimViscosity,
            gAverage(popBal_.continuousPhase().thermo().nu()())
        );

        const dimensionedScalar rhoc
        (
            "rhoc", dimDensity, gAverage(popBal_.continuousPhase().rho()())
        );

        const dimensionedScalar rhod
        (
            "rhod",
            dimDensity,
            gAverage(popBal_.sizeGroups()[1].phase().rho()())
        );

        const dimensionedScalar sigma
        (
            "sigma",
            dimForce/dimLength,
            gAverage
            (
                popBal_.sigmaWithContinuousPhase
                (
                    popBal_.sizeGroups()[1].phase()
                )()
            )
        );

        for(int m = 0; m < popBal_.sizeGroups().size(); m++)
        {
            const sizeGroup& f = popBal_.sizeGroups()[m];

            // Initial values
            dimensionedScalar uTerminal("uTerminal", dimVelocity, 0.2);
            dimensionedScalar Cd("Cd", dimless, 0.44);
            dimensionedScalar CdEllipse("CdEllipse", dimless, 1);

            dimensionedScalar Re(uTerminal*f.dSph()/nuc);
            dimensionedScalar Eo(mag(g)*mag(rhoc - rhod)*sqr(f.dSph())/sigma);

            // Scalars for the iteration
            dimensionedScalar F("F", dimForce/dimArea, 1);
            dimensionedScalar dF("dF", dimForce/dimArea/dimVelocity, 1);
            dimensionedScalar uTerminalX("uTerminalX", dimVelocity, 1.0e-05);
            dimensionedScalar ReX("ReX", dimless, Re.value());
            dimensionedScalar CdX("CdX", dimless, Cd.value());
            dimensionedScalar dCd("dCd", Cd.dimensions()/dimVelocity, Zero);

            int n = 0;

            while(mag(F.value()) >= 1.0e-05 && n++ <= 20)
            {
                Re = uTerminal*f.dSph()/nuc;

                Cd =
                    pos0(1000 - Re)*24/Re*(1 + 0.1*pow(Re, 0.75))
                  + neg(1000 - Re)*0.44;

                CdEllipse = 0.6666*sqrt(Eo);

                Cd =
                    pos0(CdEllipse - Cd)
                   *min(CdEllipse.value(), 2.66667)
                  + neg(CdEllipse - Cd)*Cd;

                F =
                    4.0/3.0*(rhoc - rhod)*mag(g)*f.dSph()
                  - rhoc*Cd*sqr(uTerminal);

                ReX = (uTerminal + uTerminalX)*f.dSph()/nuc;

                CdX =
                    pos0(1000 - ReX)
                   *24/ReX*(1 + 0.1*pow(ReX, 0.75))
                  + neg(1000 - ReX)*0.44;

                CdX =
                    pos0(CdEllipse - CdX)
                   *min(CdEllipse.value(), 2.66667)
                  + neg(CdEllipse - CdX)*CdX;

                dCd = (CdX - Cd)/uTerminalX;

                dF = -(2*rhoc*uTerminal*Cd + rhoc*sqr(uTerminal)*dCd);

                uTerminal -= F/dF;
            }

            uTerminal_.append(new dimensionedScalar("uTerminal", uTerminal));

            Cd_.append(new dimensionedScalar("Cd", Cd));
        }
    }
}


void Foam::diameterModels::coalescenceModels::LiaoCoalescence::
addToCoalescenceRate
(
    volScalarField& coalescenceRate,
    const label i,
    const label j
)
{
    const phaseModel& continuousPhase = popBal_.continuousPhase();
    const sizeGroup& fi = popBal_.sizeGroups()[i];
    const sizeGroup& fj = popBal_.sizeGroups()[j];

    dimensionedScalar dEq(2*fi.dSph()*fj.dSph()/(fi.dSph() + fj.dSph()));
    dimensionedScalar Aij(pi*0.25*sqr(fi.dSph() + fj.dSph()));

    if (turbulence_)
    {
        uRelTurb_ =
            CTurb_*sqrt(2.0)
           *sqrt(sqr(cbrt(fi.dSph())) + sqr(cbrt(fj.dSph())))
           *cbrt(popBal_.continuousTurbulence().epsilon());
    }

    if (buoyancy_)
    {
        uRelBuoy_ = CBuoy_*mag(uTerminal_[i] - uTerminal_[j]);
    }

    if (laminarShear_)
    {
        uRelShear_ = CShear_*0.5/pi*(fi.dSph() + fj.dSph())*shearStrainRate_;
    }

    // Eq. (18')
    const volScalarField collisionEfficiency
    (
        neg(kolmogorovLengthScale_ - (fi.dSph() + fj.dSph()))
       *exp
        (
          - CEff_*sqrt
            (
                continuousPhase.rho()*dEq
               /popBal_.sigmaWithContinuousPhase(fi.phase())
               *sqr(max(uRelTurb_, max(uRelBuoy_, uRelShear_)))
            )
        )
      + pos0(kolmogorovLengthScale_ - (fi.dSph() + fj.dSph()))
       *exp
        (
          - 3*continuousPhase.thermo().mu()*dEq*eddyStrainRate_
           /(4*popBal_.sigmaWithContinuousPhase(fi.phase()))
           *log
            (
                cbrt
                (
                    pi*popBal_.sigmaWithContinuousPhase(fi.phase())*sqr(dEq)
                   /(32*AH_)
                )
            )
        )
    );

    // Eq. (17)
    if (turbulence_)
    {
        coalescenceRate +=
            neg(kolmogorovLengthScale_ - (fi.dSph() + fj.dSph()))
           *CPack_*Aij*uRelTurb_*collisionEfficiency;
    }

    // Eq. (18)
    if (buoyancy_)
    {
        coalescenceRate += CPack_*0.5*Aij*uRelBuoy_*collisionEfficiency;
    }

    // Eq. (21)
    if (laminarShear_)
    {
        coalescenceRate += CPack_*0.5*Aij*uRelShear_*collisionEfficiency;
    }

    // Eq. (22)
    if (eddyCapture_)
    {
        const volScalarField uRelEddy
        (
            CEddy_*0.5/pi*(fi.dSph() + fj.dSph())*eddyStrainRate_
        );

        coalescenceRate +=
            pos0(kolmogorovLengthScale_ - (fi.dSph() + fj.dSph()))
           *CPack_*0.5*Aij*uRelEddy*collisionEfficiency;
    }

    // Eq. (25)
    if (wakeEntrainment_)
    {
        // Eq. (24) for bubble i
        const dimensionedScalar uRelWakeI
        (
            CWake_*uTerminal_[i]*cbrt(Cd_[i])
        );

        // Eq. (24) for bubble j
        const dimensionedScalar uRelWakeJ
        (
            CWake_*uTerminal_[j]*cbrt(Cd_[j])
        );

        coalescenceRate +=
            CPack_*0.125*pi
           *(
                sqr(fi.dSph())*uRelWakeI
               *pos0(fi.dSph() - 0.5*dCrit_)
               *(
                    pow6(fi.dSph() - 0.5*dCrit_)
                   /(pow6(fi.dSph() - 0.5*dCrit_) + pow6(0.5*dCrit_))
                )
              + sqr(fj.dSph())*uRelWakeJ
               *pos0(fj.dSph() - 0.5*dCrit_)
               *(
                    pow6(fj.dSph() - 0.5*dCrit_)
                   /(pow6(fj.dSph() - 0.5*dCrit_) + pow6(0.5*dCrit_))
                )
            );
    }
}


// ************************************************************************* //
