/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2018-2020 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "mixedStructuralModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace LESModels
{

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

template
<
    class BasicMomentumTransportModel,
    template<class> class EddyViscosityMomentumTransportModel,
    template<class> class StructuralMomentumTransportModel
>
mixedStructuralModel
<
    BasicMomentumTransportModel,
    EddyViscosityMomentumTransportModel,
    StructuralMomentumTransportModel
>::
mixedStructuralModel
(
    const alphaField& alpha,
    const rhoField& rho,
    const volVectorField& U,
    const surfaceScalarField& alphaRhoPhi,
    const surfaceScalarField& phi,
    const transportModel& transport,
    const word& type
)
:
    BasicStructural
    (
        alpha,
        rho,
        U,
        alphaRhoPhi,
        phi,
        transport,
        type
    )
{
    if (type == typeName)
    {
        this->printCoeffs(type);
    }
}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

template
<
    class BasicMomentumTransportModel,
    template<class> class EddyViscosityMomentumTransportModel,
    template<class> class StructuralMomentumTransportModel
>
tmp<volScalarField>
mixedStructuralModel
<
    BasicMomentumTransportModel,
    EddyViscosityMomentumTransportModel,
    StructuralMomentumTransportModel
>::
epsilon() const
{
    return BasicStructural::epsilon()
      + BasicEddyViscosity::epsilon();
}


template
<
    class BasicMomentumTransportModel,
    template<class> class EddyViscosityMomentumTransportModel,
    template<class> class StructuralMomentumTransportModel
>
tmp<volSymmTensorField>
mixedStructuralModel
<
    BasicMomentumTransportModel,
    EddyViscosityMomentumTransportModel,
    StructuralMomentumTransportModel
>::
sigma() const
{
    return BasicStructural::sigma()
      + BasicEddyViscosity::sigma();
}


template
<
    class BasicMomentumTransportModel,
    template<class> class EddyViscosityMomentumTransportModel,
    template<class> class StructuralMomentumTransportModel
>
tmp<volSymmTensorField>
mixedStructuralModel
<
    BasicMomentumTransportModel,
    EddyViscosityMomentumTransportModel,
    StructuralMomentumTransportModel
>::
devTau() const
{
    return BasicStructural::devTau()
      + BasicEddyViscosity::devTau();
}


template
<
    class BasicMomentumTransportModel,
    template<class> class EddyViscosityMomentumTransportModel,
    template<class> class StructuralMomentumTransportModel
>
tmp<fvVectorMatrix>
mixedStructuralModel
<
    BasicMomentumTransportModel,
    EddyViscosityMomentumTransportModel,
    StructuralMomentumTransportModel
>::
divDevTau
(
    volVectorField& U
) const
{
    return BasicStructural::divDevTau(U)
      + BasicEddyViscosity::divDevTau(U);
}


template
<
    class BasicMomentumTransportModel,
    template<class> class EddyViscosityMomentumTransportModel,
    template<class> class StructuralMomentumTransportModel
>
tmp<fvVectorMatrix>
mixedStructuralModel
<
    BasicMomentumTransportModel,
    EddyViscosityMomentumTransportModel,
    StructuralMomentumTransportModel
>::
divDevTau
(
    const volScalarField& rho,
    volVectorField& U
) const
{
    return BasicStructural::divDevTau(rho, U)
      + BasicEddyViscosity::divDevTau(rho, U);
}


template
<
    class BasicMomentumTransportModel,
    template<class> class EddyViscosityMomentumTransportModel,
    template<class> class StructuralMomentumTransportModel
>
void
mixedStructuralModel
<
    BasicMomentumTransportModel,
    EddyViscosityMomentumTransportModel,
    StructuralMomentumTransportModel
>::
correct()
{
    BasicStructural::correct();

    if (momentumTransportModel::debug && this->runTime_.outputTime())
    {
        tmp<volSymmTensorField> tauSgs
        (
            volSymmTensorField::New
            (
                IOobject::groupName("tauSgs", this->alphaRhoPhi_.group()),
                BasicStructural::devTau()
              + (-(this->alpha_*this->rho_*this->nut()))
               *dev(twoSymm(fvc::grad(this->U_)))
            )
        );
        Info<< "Writing " << tauSgs->name() << endl;
        tauSgs().write();

        tmp<volSymmTensorField> tauEff
        (
            this->devTau()
        );
        tauEff.ref().rename
        (
            IOobject::groupName("tauEff", this->alphaRhoPhi_.group())
        );
        Info<< "Writing " << tauEff->name() << endl;
        tauEff().write();
    }
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace LESModels
} // End namespace Foam

// ************************************************************************* //
