/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2018-2020 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::LESModels::pureStructuralModel

Description
    Abstract base class for any structural sub-grid scale model. Sub-grid stress
    tensor and derived variables are added to contributions originating from
    molecular viscosity.

SourceFiles
    pureStructuralModel.C

\*---------------------------------------------------------------------------*/

#ifndef pureStructuralModel_H
#define pureStructuralModel_H

#include "typeInfo.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace LESModels
{

/*---------------------------------------------------------------------------*\
                     Class pureStructuralModel Declaration
\*---------------------------------------------------------------------------*/

template
<
    class BasicMomentumTransportModel,
    template<class> class StructuralMomentumTransportModel
>
class pureStructuralModel
:
    public StructuralMomentumTransportModel<BasicMomentumTransportModel>
{
    // Private Data

        // Model constant for calculating nuSgs
        dimensionedScalar Ck_;


    // Private Member Functions

        // Disallow default bitwise copy construct and assignment
        pureStructuralModel(const pureStructuralModel&);
        void operator=(const pureStructuralModel&);

        //- Getter for model coefficient
        virtual dimensionedScalar Ck() const
        {
            return Ck_;
        }


public:

    typedef BasicMomentumTransportModel BasicLinearViscousStress;
    typedef StructuralMomentumTransportModel<BasicLinearViscousStress>
        BasicStructural;

    typedef typename BasicMomentumTransportModel::alphaField alphaField;
    typedef typename BasicMomentumTransportModel::rhoField rhoField;
    typedef typename BasicMomentumTransportModel::transportModel transportModel;

    //- Runtime type information
    TypeName("pureStructuralModel");


    // Constructors

        //- Construct from components
        pureStructuralModel
        (
            const alphaField& alpha,
            const rhoField& rho,
            const volVectorField& U,
            const surfaceScalarField& alphaRhoRhi,
            const surfaceScalarField& phi,
            const transportModel& transport,
            const word& type = typeName
        );


    //- Destructor
    virtual ~pureStructuralModel()
    {}


    // Member Functions

        //- Return the effective stress tensor
        virtual tmp<volSymmTensorField> devTau() const;

        //- Return the source term for the momentum equation
        virtual tmp<fvVectorMatrix> divDevTau(volVectorField& U) const;

        //- Return the source term for the momentum equation
        virtual tmp<fvVectorMatrix> divDevTau
        (
            const volScalarField& rho,
            volVectorField& U
        ) const;

        //- Correct Eddy-Viscosity and related properties
        virtual void correct();

        //- Read model coefficient
        virtual bool read();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace LESModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
#ifdef NoRepository
    #include "pureStructuralModel.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
