/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2018-2020 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "KetterlExtendedGradient.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace LESModels
{

// * * * * * * * * * * * * Protected Member Functions* * * * * * * * * * * * //

template<class BasicMomentumTransportModel>
tmp<volSymmTensorField> KetterlExtendedGradient<BasicMomentumTransportModel>::B
(
    const rhoField& rho,
    const volVectorField& U
) const
{
    volVectorField gradAlpha(gradWrapper(this->alpha_));
    gradAlpha *= pos(mag(gradAlpha) - thresholdGradAlpha_);

    volVectorField gradRho(gradWrapper(rho));
    gradRho *= pos(mag(gradRho) - thresholdGradRho_);

    volTensorField gradU(fvc::grad(U));
    gradU *= pos(mag(gradU) - thresholdGradU_);

    return
        sqr(this->delta())/12
       *symm
       (
            this->alpha_*rho*(gradU.T() & gradU)
          + this->alpha_*(U*(gradRho & gradU))
          + this->alpha_*((gradRho & gradU)*U)
          + rho*(U*(gradAlpha & gradU))
          + rho*((gradAlpha & gradU)*U)
          + (U*U)*(gradAlpha & gradRho)
        );
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

template<class BasicMomentumTransportModel>
KetterlExtendedGradient<BasicMomentumTransportModel>::KetterlExtendedGradient
(
    const alphaField& alpha,
    const rhoField& rho,
    const volVectorField& U,
    const surfaceScalarField& alphaRhoPhi,
    const surfaceScalarField& phi,
    const transportModel& transport,
    const word& type
)
:
    structuralModel<BasicMomentumTransportModel>
    (
        alpha,
        rho,
        U,
        alphaRhoPhi,
        phi,
        transport,
        type
    ),

    thresholdGradAlpha_
    (
        dimensionedScalar::lookupOrAddToDict
        (
            "thresholdGradAlpha",
            this->coeffDict_,
            inv(dimLength),
            Zero
        )
    ),

    thresholdGradRho_
    (
        dimensionedScalar::lookupOrAddToDict
        (
            "thresholdGradRho",
            this->coeffDict_,
            dimDensity/dimLength,
            Zero
        )
    ),

    thresholdGradU_
    (
        dimensionedScalar::lookupOrAddToDict
        (
            "thresholdGradU",
            this->coeffDict_,
            inv(dimTime),
            Zero
        )
    )
{}


template<class BasicMomentumTransportModel>
KetterlExtendedGradient<BasicMomentumTransportModel>::KetterlExtendedGradient
(
    const word& type,
    const alphaField& alpha,
    const rhoField& rho,
    const volVectorField& U,
    const surfaceScalarField& alphaRhoPhi,
    const surfaceScalarField& phi,
    const transportModel& transport
)
:
    structuralModel<BasicMomentumTransportModel>
    (
        type,
        alpha,
        rho,
        U,
        alphaRhoPhi,
        phi,
        transport
    ),

    thresholdGradAlpha_
    (
        dimensionedScalar::lookupOrAddToDict
        (
            "thresholdGradAlpha",
            this->coeffDict_,
            inv(dimLength),
            Zero
        )
    ),

    thresholdGradRho_
    (
        dimensionedScalar::lookupOrAddToDict
        (
            "thresholdGradRho",
            this->coeffDict_,
            dimDensity/dimLength,
            Zero
        )
    ),

    thresholdGradU_
    (
        dimensionedScalar::lookupOrAddToDict
        (
            "thresholdGradU",
            this->coeffDict_,
            inv(dimTime),
            Zero
        )
    )
{}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace LESModels
} // End namespace Foam

// ************************************************************************* //
