/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2018-2019 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Global
    setPIDDeltaT

Description
    Set time step size with PID controller to maintain target Courant number
    (targetCo). Reduction of time-step is immediate, but increase is limited to
    a fixed rate of increase (factor 1.2) to avoid unstable oscillations.
    Additionally time step size is bounded by the value from the original time
    step adjustment algorithm, which is controlled with the maximum Courant
    number (maxCo).

\*---------------------------------------------------------------------------*/

if (adjustTimeStep)
{
    // Standard time step control
    const scalar maxDeltaTFact(maxCo/(CoNum + small));
    const scalar deltaTFact
    (
        min(min(maxDeltaTFact, 1.0 + 0.1*maxDeltaTFact), 1.2)
    );
    const scalar stdDeltaT(deltaTFact*runTime.deltaTValue());

    // PID time step control
    scalar PIDDeltaT(great);
    if (targetCo < maxCo)
    {
        const scalar deltaCo(CoNum - targetCo);

        const scalar deltaTP(kP*deltaCo);

        deltaCoSum += kI*deltaCo*runTime.deltaTValue();
        const scalar deltaTI(deltaCoSum);
        Info<< "deltaCoSum = " << deltaCoSum << endl;

        const scalar deltaTD(kD*(deltaCo - deltaCo0)/runTime.deltaTValue());
        deltaCo0 = deltaCo;

        PIDDeltaT = runTime.deltaTValue() + deltaTP + deltaTI + deltaTD;
    }

    // Limit between minDeltaT and maxDeltaT
    runTime.setDeltaT
    (
        max
        (
            min
            (
                min(stdDeltaT, PIDDeltaT),
                maxDeltaT
            ),
            minDeltaT
        )
    );

    Info<< "deltaT = " <<  runTime.deltaTValue() << endl;
}

// ************************************************************************* //
