/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2019-2021 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "phaseStructureProperties.H"
#include "volFields.H"
#include "addToRunTimeSelectionTable.H"
#include "syncTools.H"
#include "surfaceFields.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace functionObjects
{
    defineTypeNameAndDebug(phaseStructureProperties, 0);
    addToRunTimeSelectionTable
    (
        functionObject,
        phaseStructureProperties, dictionary
    );
}
}

template<>
const char*
Foam::NamedEnum
<
    Foam::functionObjects::phaseStructureProperties::regionTypes,
    2
>::names[] = {"cellZone", "all"};


const Foam::NamedEnum
<
    Foam::functionObjects::phaseStructureProperties::regionTypes,
    2
> Foam::functionObjects::phaseStructureProperties::regionTypeNames_;


// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //

void Foam::functionObjects::phaseStructureProperties::setCellZoneCells()
{
    switch (regionType_)
    {
        case stCellZone:
        {
            dict().lookup("cellZoneName") >> regionName_;

            label zoneId = mesh().cellZones().findZoneID(regionName_);

            if (zoneId < 0)
            {
                FatalErrorInFunction
                    << "Unknown cell zone name: " << regionName_
                    << ". Valid cell zones are: " << mesh().cellZones().names()
                    << nl << exit(FatalError);
            }

            cellId_ = mesh().cellZones()[zoneId];
            nCells_ = returnReduce(cellId_.size(), sumOp<label>());
            break;
        }

        case stAll:
        {
            cellId_ = identity(mesh().nCells());
            nCells_ = returnReduce(cellId_.size(), sumOp<label>());
            break;
        }

        default:
        {
            FatalErrorInFunction
               << "Unknown region type. Valid region types are:"
                << regionTypeNames_ << nl << exit(FatalError);
        }
    }

    if (debug)
    {
        Pout<< "Selected region size = " << cellId_.size() << endl;
    }
}


// * * * * * * * * * * * * Protected Member Functions  * * * * * * * * * * * //

void Foam::functionObjects::phaseStructureProperties::initialise
(
    const dictionary& dict
)
{
    setCellZoneCells();

    if (nCells_ == 0)
    {
        FatalErrorInFunction
            << type() << " " << name() << ": "
            << regionTypeNames_[regionType_] << "(" << regionName_ << "):" << nl
            << "    Region has no cells" << exit(FatalError);
    }
}


void Foam::functionObjects::phaseStructureProperties::writeFileHeader
(
    const label i
)
{
    OFstream& file = this->file();

    writeHeader
    (
        file,
        "Center of gravity, velocity and volume for phase " + phaseName_
    );

    file<<"#"
        << setw(IOstream::defaultPrecision() + 7) << " Time"
        << setw(IOstream::defaultPrecision() + 8) << "X"
        << setw(IOstream::defaultPrecision() + 8) << "Y"
        << setw(IOstream::defaultPrecision() + 8) << "Z"
        << setw(IOstream::defaultPrecision() + 8) << "U"
        << setw(IOstream::defaultPrecision() + 8) << "V"
        << setw(IOstream::defaultPrecision() + 8) << "W"
        << setw(IOstream::defaultPrecision() + 8) << "Volume"
        << endl;
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::functionObjects::phaseStructureProperties::phaseStructureProperties
(
    const word& name,
    const Time& runTime,
    const dictionary& dict
)
:
    fvMeshFunctionObject(name, runTime, dict),
    logFiles(obr_, name),
    dict_(dict),
    regionType_(regionTypeNames_.read(dict.lookup("regionType"))),
    regionName_(word::null),
    nCells_(0),
    cellId_(),
    phaseName_(dict_.lookup("phaseName")),
    velocityName_(dict_.lookupOrDefault<word>("velocityName", "U")),
    threshold_(dict_.lookupOrDefault<scalar>("threshold", 0.5)),
    thickness_(dimLength, 1),
    CG_(dimLength, Zero),
    velocity_(dimVelocity, Zero),
    volume_(dimVolume, Zero)
{
    registerDimensionedObject("CG", CG_);
    registerDimensionedObject("velocity", velocity_);
    registerDimensionedObject("volume", volume_);

    read(dict);
    resetName(name);

    if (mesh().nGeometricD() == 2)
    {
        twoDimensional_ = true;

        vector plane(0,0,0);
        if (mesh().geometricD().x() == -1)
        {
            plane = vector(1,0,0);
        }
        else if (mesh().geometricD().y() == -1)
        {
            plane = vector(0,1,0);
        }
        else if (mesh().geometricD().z() == -1)
        {
            plane = vector(0,0,1);
        }

        const cell& faces = mesh().cells()[0];
        const labelList points = faces.labels(mesh().faces());
        pointField pLocal(points.size(), vector::zero);
        forAll(points, i)
        {
            const label& pointI = points[i];
            pLocal[i] = mesh().points()[pointI];
        }

        thickness_.value() =
            Foam::max(pLocal & plane) - Foam::min(pLocal & plane);

        Info<< type() << ": "
            << "Calculating area (volume) in 2D space" << nl
            << "plane " << plane << ", thickness " << thickness_ << nl << endl;
    }
    else if (mesh().nGeometricD() == 3)
    {
        twoDimensional_ = false;

        Info<< type() << ": "
            << "Calculating volume in 3D space." << nl << endl;
    }
    else
    {
        FatalErrorInFunction
            << type() << ": Only works for 2D and 3D cases."
            << exit(FatalError);
    }
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::functionObjects::phaseStructureProperties::~phaseStructureProperties()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

bool Foam::functionObjects::phaseStructureProperties::read
(
    const dictionary& dict
)
{
    fvMeshFunctionObject::read(dict);

    // No additional info to read
    initialise(dict);

    return true;
}


bool Foam::functionObjects::phaseStructureProperties::execute()
{
    return true;
}


bool Foam::functionObjects::phaseStructureProperties::write()
{
    autoPtr<volScalarField> phasePtr;
    if (obr_.foundObject<volScalarField>(phaseName_))
    {
        Info<< "    Looking up field " << phaseName_ << endl;
    }
    else
    {
        Info<< "    Reading field " << phaseName_ << endl;
        phasePtr.reset
        (
            new volScalarField
            (
                IOobject
                (
                    phaseName_,
                    mesh_.time().timeName(),
                    mesh_,
                    IOobject::MUST_READ,
                    IOobject::NO_WRITE
                ),
                mesh_
            )
        );
    }

    const volScalarField& alpha =
    (
         phasePtr.valid()
       ? phasePtr()
       : obr_.lookupObject<volScalarField>(phaseName_)
    );


    autoPtr<volVectorField> UPtr;
    if (obr_.foundObject<volVectorField>(velocityName_))
    {
        Info<< "    Looking up field " << velocityName_ << endl;
    }
    else
    {
        Info<< "    Reading field " << velocityName_ << endl;
        UPtr.reset
        (
            new volVectorField
            (
                IOobject
                (
                    velocityName_,
                    mesh_.time().timeName(),
                    mesh_,
                    IOobject::MUST_READ,
                    IOobject::NO_WRITE
                ),
                mesh_
            )
        );
    }

    const volVectorField& U =
    (
         UPtr.valid()
       ? UPtr()
       : obr_.lookupObject<volVectorField>(velocityName_)
    );

    scalarField iPhase(filterField(alpha));
    scalarField iV(filterField(mesh().V()));
    vectorField iC(filterField(mesh().C()));
    vectorField iU(filterField(U));
    scalarField iA(mesh().nCells(), Zero);

    // Combine onto master
    combineFields(iPhase);
    combineFields(iC);
    combineFields(iV);
    combineFields(iU);

    scalar phaseSum    = sum(scalarField(iPhase*iV));
    vector phaseByCSum = sum(vectorField(iPhase*iC*iV));
    vector phaseByUSum = sum(vectorField(iPhase*iU*iV));

    if (Pstream::master())
    {
        CG_.value() = phaseByCSum/max(phaseSum, small);
        velocity_.value() = phaseByUSum/max(phaseSum, small);
        volume_.value() = phaseSum/thickness_.value();
    }

    Pstream::scatter(CG_);
    Pstream::scatter(velocity_);
    Pstream::scatter(volume_);

    registerDimensionedObject("CG", CG_);
    registerDimensionedObject("velocity", velocity_);
    registerDimensionedObject("volume", volume_);

    unsigned int w = IOstream::defaultPrecision() + 7;

    logFiles::write();

    if (Pstream::master())
    {
        file()  << setw(w) << obr_.time().value();

        file()  << ' ' << setw(w) << CG_.value().x()
                << ' ' << setw(w) << CG_.value().y()
                << ' ' << setw(w) << CG_.value().z()
                << ' ' << setw(w) << velocity_.value().x()
                << ' ' << setw(w) << velocity_.value().y()
                << ' ' << setw(w) << velocity_.value().z()
                << ' ' << setw(w) << volume_.value()
                << endl;
    }

    Log << nl
        << "Properties of phase " << phaseName_ << ": " << nl
        << "    Center of gravity: " << CG_.value() << nl
        << "    Velocity:          " << velocity_.value() << nl
        << "    Volume:            " << volume_.value() << nl
        << endl;

    return true;
}


// ************************************************************************* //
