/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2019-2021 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "trackPhaseStructures.H"
#include "addToRunTimeSelectionTable.H"
#include "syncTools.H"

#include "cyclicFvPatchFields.H"
#include "cyclicAMIFvPatchFields.H"
#include "processorCyclicFvPatch.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
    namespace functionObjects
    {
        defineTypeNameAndDebug(trackPhaseStructures, 0);

        addToRunTimeSelectionTable
        (
            functionObject,
            trackPhaseStructures,
            dictionary
        );
    }
}


// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //

Foam::Map<Foam::label>
Foam::functionObjects::trackPhaseStructures::findPatchRegions
(
    const regionSplit& regions
) const
{
    // Count number of patch faces (just for initial sizing)
    const labelHashSet patchIDs(mesh_.boundaryMesh().patchSet(patchNames_));

    label nPatchFaces = 0;
    forAllConstIter(labelHashSet, patchIDs, iter)
    {
        nPatchFaces += mesh_.boundaryMesh()[iter.key()].size();
    }


    Map<label> patchRegions(nPatchFaces);
    forAllConstIter(labelHashSet, patchIDs, iter)
    {
        const polyPatch& pp = mesh_.boundaryMesh()[iter.key()];

        // Collect all regions on the patch
        const labelList& faceCells = pp.faceCells();

        forAll(faceCells, i)
        {
            patchRegions.insert
            (
                regions[faceCells[i]],
                Pstream::myProcNo()     // dummy value
            );
        }
    }

    // Make sure all the processors have the same set of regions
    Pstream::mapCombineGather(patchRegions, minEqOp<label>());
    Pstream::mapCombineScatter(patchRegions);

    return patchRegions;
}


Foam::boolList Foam::functionObjects::trackPhaseStructures::getBlockedFaces
(
    const volScalarField& vsf
) const
{
    boolList blockedFace(mesh_.nFaces(), false);

    {
        for (label facei = 0; facei < mesh_.nInternalFaces(); facei++)
        {
            scalar ownVal = vsf[mesh_.faceOwner()[facei]];
            scalar neiVal = vsf[mesh_.faceNeighbour()[facei]];

            if
            (
                (ownVal < threshold_ && neiVal > threshold_)
             || (ownVal > threshold_ && neiVal < threshold_)
            )
            {
                blockedFace[facei] = true;
            }
        }

        // Block coupled faces
        forAll(vsf.boundaryField(), patchi)
        {
            const fvPatchScalarField& fvp(vsf.boundaryField()[patchi]);

            if (fvp.coupled())
            {
                tmp<scalarField> townFld(fvp.patchInternalField());
                const scalarField& ownFld = townFld();

                label start = fvp.patch().patch().start();

                // Cut at cyclic patch
                if
                (
                    cutAtCyclicPatch_
                 && (
                        isType<cyclicFvPatch>(fvp.patch())
                     || isType<cyclicAMIFvPatch>(fvp.patch())
                     || isType<processorCyclicFvPatch>(fvp.patch())
                    )
                )
                {
                    forAll(ownFld, i)
                    {
                        blockedFace[start+i] = true;
                    }
                }
                else
                {
                    tmp<scalarField> tnbrFld(fvp.patchNeighbourField());
                    const scalarField& nbrFld = tnbrFld();

                    forAll(ownFld, i)
                    {
                        scalar ownVal = ownFld[i];
                        scalar neiVal = nbrFld[i];

                        if
                        (
                            (ownVal < threshold_ && neiVal > threshold_)
                         || (ownVal > threshold_ && neiVal < threshold_)
                        )
                        {
                            blockedFace[start+i] = true;
                        }
                    }
                }
            }
        }
    }

    return blockedFace;
}


void Foam::functionObjects::trackPhaseStructures::updateRegionProperties
(
    const regionSplit& regions
)
{
    allRegionVolume_ = regionSum(regions, mesh_.V());

    allRegionCenterOfGravity_.clear();
    allRegionVelocity_.clear();

    if (allRegionVolume_.size())
    {
        autoPtr<volVectorField> UPtr;
        const volVectorField& U(lookupOrRead<volVectorField>(UPtr, UName_));

        forAllIter(Map<scalar>, allRegionVolume_, vIter)
        {
            allRegionVolume_[vIter.key()] /= thickness_;

            if
            (
                allRegionVolume_[vIter.key()] < minVol_
             || allRegionVolume_[vIter.key()] > maxVol_
            )
            {
                continue;
            }

            scalarField weightField(mesh_.nCells(), Zero);
            forAll(weightField, celli)
            {
                if (allRegionVolume_.find(regions[celli]) == vIter)
                {
                    weightField[celli] = 1.0;
                }
            }

            vectorField cellCenters(mesh_.C());
            scalarField cellVolumes(mesh_.V());
            vectorField cellVelocities(U.primitiveField());
            scalarField cellAreas(mesh_.nCells(), Zero);

            // Combine onto master
            combineFields(weightField);
            combineFields(cellCenters);
            combineFields(cellVolumes);
            combineFields(cellVelocities);

            if (Pstream::master())
            {
                scalar phaseSum(sum(scalarField(weightField*cellVolumes)));
                vector phaseByCSum
                (
                    sum(vectorField(weightField*cellCenters*cellVolumes))
                );
                vector phaseByUSum
                (
                    sum(vectorField(weightField*cellVelocities*cellVolumes))
                );

                allRegionCenterOfGravity_.insert
                (
                    vIter.key(),
                    phaseByCSum/phaseSum
                );

                allRegionVelocity_.insert
                (
                    vIter.key(),
                    phaseByUSum/phaseSum
                );
            }
        }

        Pstream::mapCombineScatter(allRegionCenterOfGravity_);
        Pstream::mapCombineScatter(allRegionVelocity_);
    }
}


void Foam::functionObjects::trackPhaseStructures::updateTrackedIndices
(
    const regionSplit& regions,
    const Map<vector>& allRegionCenterOfGravity0
)
{
    const Map<label> patchRegions(findPatchRegions(regions));

    if (Pstream::master())
    {
        // Save old indices
        Map<label> allRegionIndex0(0);
        if (allRegionIndex_.size())
        {
            allRegionIndex0 = allRegionIndex_;
            allRegionIndex_.clear();
        }

        forAllIter(Map<vector>, allRegionCenterOfGravity_, cIter)
        {
            const label regionI = cIter.key();

            // Exclude liquid core region
            // or regions with volume greater than maxVol
            // or regions with volume smaller than minVol
            // or regions, which are already in map
            if
            (
                patchRegions.found(regionI)
             || allRegionVolume_[regionI] > maxVol_
             || allRegionVolume_[regionI] < minVol_
             || allRegionIndex_.found(regionI)
            )
            {
                continue;
            }

            if
            (
                allRegionCenterOfGravity0.size()
             && allRegionIndex0.size()
            )
            {
                const vector cog(allRegionCenterOfGravity_[regionI]);

                scalar minDist(great);
                label c0key(-1);

                // Find closest of old structures
                forAllConstIter(Map<vector>, allRegionCenterOfGravity0, c0Iter)
                {
                    const vector cog0(allRegionCenterOfGravity0[c0Iter.key()]);
                    const scalar dist(mag(cog - cog0));

                    if
                    (
                        dist < minDist
                     && dist < tolerance_
                     && allRegionIndex0.found(c0Iter.key())
                    )
                    {
                        minDist = dist;
                        c0key = c0Iter.key();
                    }
                }

                if (c0key >= 0)
                {
                    allRegionIndex_.insert(regionI, allRegionIndex0[c0key]);
                }
                else
                {
                    allRegionIndex_.insert(regionI, index_);
                    ++index_;
                }
            }
            else
            {
                allRegionIndex_.insert(regionI, index_);
                ++index_;
            }
        }
    }

    Pstream::mapCombineScatter(allRegionIndex_);
}


void Foam::functionObjects::trackPhaseStructures::writeFileHeader
(
    const label i
)
{
    OFstream& file = this->file(i);

    writeHeader(file, "Tracked structure properties");
    file<<"# "
        << setw(IOstream::defaultPrecision() + 6) << "ID"
        << setw(IOstream::defaultPrecision() + 8) << "Time"
        << setw(IOstream::defaultPrecision() + 8) << "X"
        << setw(IOstream::defaultPrecision() + 8) << "Y"
        << setw(IOstream::defaultPrecision() + 8) << "Z"
        << setw(IOstream::defaultPrecision() + 8) << "U"
        << setw(IOstream::defaultPrecision() + 8) << "V"
        << setw(IOstream::defaultPrecision() + 8) << "W"
        << "Volume"
        << endl;
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::functionObjects::trackPhaseStructures::trackPhaseStructures
(
    const word& name,
    const Time& runTime,
    const dictionary& dict
)
:
    fvMeshFunctionObject(name, runTime, dict),
    logFiles(obr_, name),
    file_(obr_, name),
    alphaName_("alpha"),
    UName_("U"),
    patchNames_(0),
    threshold_(0.5),
    tolerance_(great),
    maxVol_(great),
    minVol_(0),
    thickness_(1),
    writeIndexField_(false),
    cutAtCyclicPatch_(true),
    allRegionIndex_(0),
    allRegionCenterOfGravity_(0),
    allRegionVelocity_(0),
    allRegionVolume_(0),
    index_(1),
    regions_(0)
{
    read(dict);
    resetName(name);

    if (mesh_.nGeometricD() == 2)
    {
        twoDimensional_ = true;

        vector plane(0,0,0);
        if (mesh_.geometricD().x() == -1)
        {
            plane = vector(1,0,0);
        }
        else if (mesh_.geometricD().y() == -1)
        {
            plane = vector(0,1,0);
        }
        else if (mesh_.geometricD().z() == -1)
        {
            plane = vector(0,0,1);
        }

        const cell& faces = mesh_.cells()[0];
        const labelList points = faces.labels(mesh_.faces());
        pointField pLocal(points.size(), vector::zero);
        forAll(points, i)
        {
            const label& pointI = points[i];
            pLocal[i] = mesh_.points()[pointI];
        }

        thickness_ = Foam::max(pLocal & plane) - Foam::min(pLocal & plane);

        Info<< type() << ": "
            << "Calculating area (volume) in 2D space" << nl
            << "plane " << plane << ", thickness " << thickness_ << nl << endl;
    }
    else if (mesh_.nGeometricD() == 3)
    {
        twoDimensional_ = false;

        Info<< type() << ": "
            << "Calculating volume in 3D space." << nl << endl;
    }
    else
    {
        FatalErrorInFunction
            << type() << ": Only works for 2D and 3D cases."
            << exit(FatalError);
    }
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::functionObjects::trackPhaseStructures::~trackPhaseStructures()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

bool Foam::functionObjects::trackPhaseStructures::read(const dictionary& dict)
{
    if (fvMeshFunctionObject::read(dict))
    {
        dict.readIfPresent("alpha", alphaName_);
        dict.readIfPresent("U", UName_);
        dict.readIfPresent("excludePatches", patchNames_);
        dict.readIfPresent("threshold", threshold_);
        dict.readIfPresent("tolerance", tolerance_);
        dict.readIfPresent("maxVolume", maxVol_);
        dict.readIfPresent("minVolume", minVol_);
        dict.readIfPresent("writeIndexField", writeIndexField_);
        dict.readIfPresent("cutAtCyclicPatch", cutAtCyclicPatch_);

        if (dict.found("coordinateSystem"))
        {
            coordSysPtr_ =
                coordinateSystem::New(obr_, dict.subDict("coordinateSystem"));

            Info<< "Transforming all vectorFields with coordinate system "
                << coordSysPtr_().name() << endl;
        }

        return true;
    }
    else
    {
        return false;
    }
}


bool Foam::functionObjects::trackPhaseStructures::execute()
{
    Info<< type() << " " << name() << " execute:" << nl;

    autoPtr<volScalarField> alphaPtr;
    const volScalarField& alpha
    (
        lookupOrRead<volScalarField>(alphaPtr, alphaName_)
    );

    // Split into regions
    regionSplit regions(mesh_, getBlockedFaces(alpha));
    Info<< "    Determined " << regions.nRegions() << " regions" << endl;

    if (writeIndexField_)
    {
        regions_ = regions;
    }

    // Save center of gravity
    Map<vector> allRegionCenterOfGravity0(0);
    if (allRegionCenterOfGravity_.size())
    {
        allRegionCenterOfGravity0 = allRegionCenterOfGravity_;
    }

    updateRegionProperties(regions);

    updateTrackedIndices(regions, allRegionCenterOfGravity0);

    return true;
}


bool Foam::functionObjects::trackPhaseStructures::write()
{
    Info<< type() << " " << name() << " write:" << nl;

    // Write data to file
    logFiles::write();

    unsigned int w = IOstream::defaultPrecision() + 7;

    if (Pstream::master())
    {
        forAllIter(Map<label>, allRegionIndex_, iIter)
        {
            const label regionI((allRegionIndex_.find(iIter.key())).key());

            file()  << setw(w) << allRegionIndex_[iIter.key()]
                << ' ' << setw(w) << obr_.time().value()
                << ' ' << setw(w) << allRegionCenterOfGravity_[regionI].x()
                << ' ' << setw(w) << allRegionCenterOfGravity_[regionI].y()
                << ' ' << setw(w) << allRegionCenterOfGravity_[regionI].z()
                << ' ' << setw(w) << allRegionVelocity_[regionI].x()
                << ' ' << setw(w) << allRegionVelocity_[regionI].y()
                << ' ' << setw(w) << allRegionVelocity_[regionI].z()
                << ' ' << allRegionVolume_[regionI]
                << endl;
        }
    }

    // Write out global index field
    if (writeIndexField_)
    {
        volScalarField globalIndex
        (
            IOobject
            (
                alphaName_ + "_globalIndex",
                obr_.time().timeName(),
                obr_,
                IOobject::NO_READ
            ),
            mesh_,
            dimensionedScalar(dimless, Zero)
        );

        forAll(globalIndex, celli)
        {
            const label regionI = regions_[celli];

            if (allRegionIndex_.found(regionI))
            {
                globalIndex[celli] = allRegionIndex_[regionI];
            }
        }
        globalIndex.correctBoundaryConditions();

        Info<< "    Writing global index field to " << globalIndex.name()
            << endl;
        globalIndex.write();
    }

    return true;
}


// ************************************************************************* //
