/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2019-2021 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::functionObjects::trackPhaseStructures

Description
    This function object tracks multiple phase structures (disconnected regions)
    and calculate center of gravity, velocity and volume of each of them.
    Structures are detected from alpha field of given name and with given
    threshold criterion. Structures, which are connected to a patch out of a
    given list, may be excluded from detection. This function object keeps track
    of structures at every execution (executionTime) and only if the physical
    distance a structure travelled since the last execution is below a given
    tolerance. Furthermore structures may be excluded if they have a volume
    below given minimum or maximum value. The field of tracked indices may be
    written at write time. The velocity of structures is determined from a
    velocity field of a given name.

    Example of function object specification:
    \verbatim
    bubbles
    {
        type                trackPhaseStructures;
        libs                ("libHZDRfieldFunctionObjects.so");
        executeControl      runTime;
        executeInterval     1e-3;
        writeControl        runTime;
        writeInterval       0.1;
        log                 true;
        ...
        alpha               alpha.airC;
        U                   U.airC;
        excludePatches      (outlet inlet1 inlet2);
        threshold           0.8;
        tolerance           0.005;
        maxVolume           4e-6;
        minVolume           1e-6;
        writeIndexField     true;
        cutAtCyclicPatch    true;
    }
    \endverbatim

Usage
    \table
        Property       | Description                  | Required| Default value
        type           | type name: trackPhaseStructures | yes   |
        alpha          | name of alpha field             | no    | alpha
        U              | name of velocity field          | no    | U
        excludePatches | boundaries to exclude           | no    | none
        threshold      | value for detection             | no    | 0.5
        tolerance      | max distance to track           | no    | great
        maxVolume      | maximum volume                  | no    | great
        minVolume      | minimum volume                  | no    | 0
        writeIndexField| write out index field           | no    | false
        cutAtCyclicPatch| structures are split by cyclic patches | no | true
    \endtable

See also
    Foam::functionObjects::fvMeshFunctionObject
    Foam::functionObjects::logFiles

SourceFiles
    trackPhaseStructures.C

\*---------------------------------------------------------------------------*/

#ifndef functionObjects_trackPhaseStructures_H
#define functionObjects_trackPhaseStructures_H

#include "fvMeshFunctionObject.H"
#include "logFiles.H"
#include "writer.H"
#include "volFieldsFwd.H"
#include "coordinateSystem.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// Forward declaration of classes
class regionSplit;

namespace functionObjects
{

/*---------------------------------------------------------------------------*\
                    Class trackPhaseStructures Declaration
\*---------------------------------------------------------------------------*/

class trackPhaseStructures
:
    public fvMeshFunctionObject,
    public logFiles
{
    // Private Data

        //- Result file
        writeFile file_;

        //- Name of alpha field
        word alphaName_;

        //- Names of velocity field
        word UName_;

        //- Patches to walk from
        wordReList patchNames_;

        //- Clipping value
        scalar threshold_;

        //- Tolerance for distance a structure has moved over time
        scalar tolerance_;

        //- Maximum structure volume
        scalar maxVol_;

        //- Minimum structure volume
        scalar minVol_;

        //- Two dimensional domain
        bool twoDimensional_;

        //- For 2D cases: thickness of domain in 3rd direction
        scalar thickness_;

        //- Switch for calculating and writing global index as field
        bool writeIndexField_;

        //- Ignore cyclic patches and treat them as region boundary
        bool cutAtCyclicPatch_;

        //- Output formatter to write
        autoPtr<writer<scalar>> formatterPtr_;

        //- Optional coordinate system
        autoPtr<coordinateSystem> coordSysPtr_;

        //- Indices of regions
        Map<label> allRegionIndex_;

        //- Center of gravity of all regions
        Map<vector> allRegionCenterOfGravity_;

        //- Velocity of all regions
        Map<vector> allRegionVelocity_;

        //- Volume (2D: area) of all regions
        Map<scalar> allRegionVolume_;

        //- Global index number for all tracked structures
        label index_;

        //- Regions as result from splitRegion
        labelList regions_;


    // Private Member Functions

        //- Combine fields from all processor domains into single field
        template<class Type>
        void combineFields(Field<Type>& field);

        //- Try to lookup object, read if not in obr
        template<class Type>
        const Type& lookupOrRead(autoPtr<Type>&, const word& name) const;

        //- Sum up field over all regions
        template<class Type>
        Map<Type> regionSum(const regionSplit&, const Field<Type>&) const;

        //- Mark all regions starting at patches
        Map<label> findPatchRegions(const regionSplit&) const;

        //- Determine blocked faces
        boolList getBlockedFaces(const volScalarField&) const;

        //- Update center of gravity, velocity and volume for all regions
        void updateRegionProperties(const regionSplit&);

        //- Update global indices
        void updateTrackedIndices(const regionSplit&, const Map<vector>&);

        //- Output file header information
        virtual void writeFileHeader(const label i);

        //- Disallow default bitwise copy construct
        trackPhaseStructures(const trackPhaseStructures&);

        //- Disallow default bitwise assignment
        void operator=(const trackPhaseStructures&);


public:

    //- Runtime type information
    TypeName("trackPhaseStructures");


    // Constructors

        //- Construct for given objectRegistry and dictionary.
        //  Allow the possibility to load fields from files
        trackPhaseStructures
        (
            const word& name,
            const Time& runTime,
            const dictionary&
        );


    // Destructor

        virtual ~trackPhaseStructures();


    // Member Functions

        //- Read the trackPhaseStructures data
        virtual bool read(const dictionary&);

        //- Detect structures and keep track
        virtual bool execute();

        //- Write out individual data of phase structures and write index field
        virtual bool write();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace functionObjects
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#ifdef NoRepository
    #include "trackPhaseStructuresTemplates.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
